--
-- multiport_ocp_if2_b64.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- multiport_ocp_if.vhd - OCP_IP interface giving 2 Masters
-- control over single slave memory
--
--
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library Work;
use work.ocpad.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity MULTIPORT_OCP_IF2_b64 is
  generic (
    round_robin_arb : boolean := false;
    use_timeout  : boolean := false;
    timeout_bits : integer := 16);
  port(
    -- OCP Interface (Block Data Flow Profile)
    ocp_m2s_memif : out ocpad_burst64_m2sT;
    ocp_s2m_memif : in  ocpad_burst64_s2mT;

    ocp_m2s0 : in  ocpad_burst64_m2sT;
    ocp_s2m0 : out ocpad_burst64_s2mT;
    ocp_m2s1 : in  ocpad_burst64_m2sT;
    ocp_s2m1 : out ocpad_burst64_s2mT;
    status   : out std_logic_vector(3 downto 0);
    Clk : in std_logic);

  
end MULTIPORT_OCP_IF2_b64;

architecture mixed of MULTIPORT_OCP_IF2_b64 is

  signal active_port     : std_logic_vector(1 downto 0) := "00";
  signal busy            : std_logic                    := '0';
  signal writing         : std_logic                    := '0';
  signal reading         : std_logic                    := '0';
  signal ocp_m2s_memif_i : ocpad_burst64_m2sT  := ocpad_burst64_default_m2s;

  signal timeout_counter : std_logic_vector(timeout_bits-1 downto 0);
  signal err,error_flag,rst : std_logic := '0';
 
  
  attribute max_fanout : string;
  attribute max_fanout of active_port : signal is "32";
  
begin

  status <= writing & reading & active_port;
  
  
  ocp_m2s_memif <= ocp_m2s_memif_i;

  ocp_m2s_memif_i <= ocp_m2s0 when active_port = "00" else
                     ocp_m2s1;
  err <= error_flag or rst;

  process (active_port,err,ocp_s2m_memif)
  begin  -- process
    case active_port is
      when "00" =>
        ocp_s2m0 <= ocp_s2m_memif;
        ocp_s2m1 <= ocpad_burst64_default_s2m;
      when "01" =>
        ocp_s2m0 <= ocpad_burst64_default_s2m;
        ocp_s2m1 <= ocp_s2m_memif;
      when others =>
        ocp_s2m0 <= ocpad_burst64_default_s2m;
        ocp_s2m1 <= ocpad_burst64_default_s2m;
    end case;
    ocp_s2m0.error <= err;
    ocp_s2m1.error <= err;
  end process;

  busy <= writing or reading;

  switch_port : process (clk)
  begin  -- process switch_port
    if clk'event and clk = '1' then     -- rising clock edge
      if active_port = "00" then
        if ocp_m2s0.Cmd /= ocp_Cmd_WRITE and ocp_m2s0.Cmd /= ocp_Cmd_READ then
          if busy = '0' then
            if ocp_m2s1.Cmd = ocp_Cmd_WRITE or ocp_m2s1.Cmd = ocp_Cmd_READ  then
              active_port <= "01";
            end if;
          end if;
        end if;
      else
        if ocp_m2s1.Cmd /= ocp_Cmd_WRITE and ocp_m2s1.Cmd /= ocp_Cmd_READ then
          if busy = '0' then
            if ocp_m2s0.Cmd = ocp_Cmd_WRITE or ocp_m2s0.Cmd = ocp_Cmd_READ then
              active_port <= "00";
            end if;
          end if;
        end if;    
      end if;
      if ocp_m2s_memif_i.Cmd = ocp_Cmd_WRITE or ocp_m2s_memif_i.Cmd = ocp_Cmd_WRITENONPOST then
        writing <= '1';
      elsif ocp_m2s_memif_i.DataLast = '1' and ocp_m2s_memif_i.DataValid = '1' and ocp_s2m_memif.DataAccept = '1' then
        writing <= '0';
        if round_robin_arb then
          if active_port = "01" then
            active_port <= "00";
          else
            active_port <= active_port +1;
          end if;
        end if;
      end if;

      if ocp_m2s_memif_i.Cmd = ocp_Cmd_READ then
        reading <= '1';
      elsif ocp_s2m_memif.RespLast = '1' and ocp_m2s_memif_i.RespAccept = '1' then
        reading <= '0';
        if round_robin_arb then
          if active_port = "01" then
            active_port <= "00";
          else
            active_port <= active_port +1;
          end if;
        end if;
      end if;
      
      if ocp_m2s0.Reset_n = '0' or ocp_m2s1.Reset_n = '0' then  
        writing <= '0';
        reading <= '0';
        active_port <= "00";
        rst <= '1';
      else
        rst <= '0';
      end if;

      if use_timeout then
        if writing = '0' and reading = '0' then
          timeout_counter <= (others => '0');
          error_flag <= '0';
        else
          timeout_counter <= timeout_counter+1;
          if AND_reduce(timeout_counter) = '1' then
            writing <= '0';
            reading <= '0';
            active_port <= "00";
            error_flag <= '1';
          end if;
        end if;      
      end if;
      
    end if;
  end process switch_port;
  


end mixed;
