--
-- ocp2slavebus64.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
--
-- Simple interface bridging from OCP to a simple bus


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.ocpad.all;
use work.slave_bus_if_types.all;

entity ocp2slavebus64 is
  port (
    -- OCP Interface
    ocp_clk : in std_logic;
    ocp_rst : in std_logic;
    ocp_m2s : in ocpad_register64_m2sT;
    ocp_s2m : out ocpad_register64_s2mT;
    -- Slave bus Interface
    slave_bus_outputs       : out slave_bus_outputs_type;
    slave_bus_inputs        : in  slave_bus_inputs_type);

end ocp2slavebus64;

architecture rtl of ocp2slavebus64 is

  signal rd : std_logic := '0';
  signal addr : std_logic_vector(21 downto 0);
  signal wr : std_logic := '0';
  signal write_reg : std_logic := '0';
  signal data : std_logic_vector(63 downto 0) := (others => '0');
  signal rd_req : std_Logic := '0'; 

  signal be : std_logic_vector(7 downto 0);
  signal wcmd : std_logic;
  signal rcmd : std_logic;

  signal SCmdAccept : std_logic := '0';
  signal SResp : std_logic_vector(1 downto 0) := "00";
  signal SData : std_logic_vector(63 downto 0) := (others => '0');
 

 
  
begin  -- rtl

  process (ocp_clk)
  begin  -- process
    if ocp_clk'event and ocp_clk = '1' then  -- rising clock edge
      rd_req <= '0';
      if (ocp_m2s.Cmd = "001" or ocp_m2s.Cmd = "010") and rd_req = '0' then
        if ocp_m2s.Cmd = "010" then
           rd_req <= '1';
        end if;
        addr <= ocp_m2s.Addr(21 downto 0);
      end if;
     

      rd <= rd_req;      
      if ocp_m2s.Cmd = "001" then
        write_reg <= '1';
      else
        if ocp_m2s.Cmd = "000" then
          write_reg <= '0';
        end if;
      end if;

      if ocp_m2s.Cmd = "001" then
        data <= ocp_m2s.data;
        be <= ocp_m2s.databyteen;
      end if;

      wr <= to_std_logic(ocp_m2s.Cmd = "001") and (write_reg or wcmd) and not wr;

      SCmdAccept <= (wcmd or rcmd) and not SCmdAccept;
      SData <= slave_bus_inputs.q;

      SResp(0) <= slave_bus_inputs.qv;
      SResp(1) <= '0';
 
    end if;
  end process;

  wcmd <= '1' when ocp_m2s.Cmd = "001" else '0';
  rcmd <= '1' when ocp_m2s.Cmd = "010" else '0';

  
  ocp_s2m.CmdAccept       <=  SCmdAccept;      
  ocp_s2m.Resp            <=  SResp;           
  ocp_s2m.Data            <=  SData;           
  

  slave_bus_outputs.a <= addr;
  slave_bus_outputs.wr <= wr;
  slave_bus_outputs.rd <= rd;
  slave_bus_outputs.d <= data;
  slave_bus_outputs.be <= be;
  
end rtl;
