--
-- ocp_chipscope32.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- Chipscope connected OCP monitor
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library Work;
use work.ocpad.all;

entity ocp_chipscope32 is
  port(
    -- OCP Interface (Block Data Flow Profile)
    ocp_m2s : in ocpad_burst32_m2sT;
    ocp_s2m : in ocpad_burst32_s2mT;
    Clk : in std_logic);

  
end ocp_chipscope32;

architecture rtl of ocp_chipscope32 is
  
   -------------------------------------------------------------------
  --
  --  ICON core component declaration
  --
  -------------------------------------------------------------------
  component icon
    port
    (
      control0    :   out std_logic_vector(35 downto 0)
    );
  end component;


   -------------------------------------------------------------------
  --
  --  ILA core component declaration
  --
  -------------------------------------------------------------------
  component ila
    port
    (
      control     : in    std_logic_vector(35 downto 0);
      clk         : in    std_logic;
      trig0       : in    std_logic_vector(71 downto 0)
    );
  end component;


  -------------------------------------------------------------------
  --
  --  ILA core signal declarations
  --
  -------------------------------------------------------------------
  signal control    : std_logic_vector(35 downto 0);
  signal data       : std_logic_vector(63 downto 0);
  signal trig0      : std_logic_vector(71 downto 0);
  
begin
 -------------------------------------------------------------------
  --
  --  ICON core instance
  --
  -------------------------------------------------------------------
  i_icon : icon
    port map
    (
      control0    => control
    );

   -------------------------------------------------------------------
  --
  --  ILA core instance
  --
  -------------------------------------------------------------------
  i_ila : ila
    port map
    (
      control   => control,
      clk       => clk,
      trig0     => trig0
    );

  trig0(2 downto 0) <=  ocp_m2s.Cmd;
  trig0(3) <= ocp_m2s.DataValid;
  trig0(4) <= ocp_s2m.CmdAccept;
  trig0(5) <= ocp_s2m.DataAccept;
  trig0(7 downto 6) <= ocp_s2m.Resp;

  data(15 downto 0) <= ocp_m2s.Addr(15 downto 0);
  data(31 downto 16) <= ocp_m2s.Data(15 downto 0);
  data(47 downto 32) <= ocp_s2m.Data(15 downto 0);
  data(55 downto 48) <= ocp_m2s.BurstLength(7 downto 0);
  data(56) <= ocp_m2s.BurstSingleReq;
  data(57) <= ocp_m2s.RespAccept;
  data(58) <= ocp_m2s.DataLast;
  data(59) <= ocp_s2m.RespLast;
  data(60) <= ocp_s2m.Error;
  data(61) <= ocp_m2s.Reset_n;
  data(62) <= ocp_m2s.DataByteEn(0);
  data(63) <= ocp_m2s.DataByteEn(2);

  trig0(71 downto 8) <= data;
  
  
  

  
end rtl;
  
