--
-- ocp_demux_b32.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- ocp_demux_b32.vhd - OCP_IP interface for splitting the OCP address space
-- evenly among a number of slaves.
--
--
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library Work;
use work.ocpad.all;

library UNISIM;
use UNISIM.VCOMPONENTS.all;

entity ocp_demux_b32 is
  generic (
    number_of_slaves  : integer := 2;
    demux_address_lsb : integer := 16);
  port(
    clk : in std_logic;
    master_m2s : in ocpad_burst32_m2sT;
    master_s2m : out  ocpad_burst32_s2mT;
    slaves_m2s : out ocpad_burst32_m2sT_array(0 to number_of_slaves-1);
    slaves_s2m : in  ocpad_burst32_s2mT_array(0 to number_of_slaves-1));

end ocp_demux_b32;

architecture mixed of ocp_demux_b32 is

  signal active_port     : std_logic_vector(32-demux_address_lsb-1 downto 0) := (others => '0');
  signal master_s2m_i : ocpad_burst32_s2mT;
  
begin

  gen_slave_m2s_ports: for i in 0 to number_of_slaves-1 generate
    slaves_m2s(i).Cmd <= master_m2s.Cmd when CONV_INTEGER(master_m2s.Addr(31 downto demux_address_lsb)) = i else ocp_Cmd_IDLE;
    slaves_m2s(i).Addr <= EXT(master_m2s.Addr(demux_address_lsb-1 downto 0),32);
    slaves_m2s(i).BurstLength <= master_m2s.BurstLength;
    slaves_m2s(i).BurstSingleReq <= master_m2s.BurstSingleReq;
    slaves_m2s(i).Data <= master_m2s.Data;
    slaves_m2s(i).DataByteEn <= master_m2s.DataByteEn;
    slaves_m2s(i).DataLast <= master_m2s.DataLast;
    slaves_m2s(i).DataValid <= master_m2s.DataValid;
    slaves_m2s(i).RespAccept <= master_m2s.RespAccept;
    slaves_m2s(i).Reset_n <= master_m2s.Reset_n;   
  end generate gen_slave_m2s_ports;

  set_active_port: process (clk)
  begin  -- process set_active_port
    if clk'event and clk = '1' then  -- rising clock edge
      if master_m2s.Cmd /= ocp_Cmd_IDLE and master_s2m_i.CmdAccept = '1' then
        active_port <= master_m2s.Addr(31 downto demux_address_lsb);
      end if;
    end if;
  end process set_active_port;

  gen_s2m_ports: for i in 0 to number_of_slaves-1 generate
    master_s2m_i.CmdAccept <= slaves_s2m(i).CmdAccept when CONV_INTEGER(master_m2s.Addr(31 downto demux_address_lsb)) = i else 'Z';
    master_s2m_i.DataAccept <= slaves_s2m(i).DataAccept when CONV_INTEGER(active_port) = i else 'Z';
    master_s2m_i.Data <= slaves_s2m(i).Data when CONV_INTEGER(active_port) = i else (others => 'Z');
    master_s2m_i.Resp <= slaves_s2m(i).Resp when CONV_INTEGER(active_port) = i else (others => 'Z');
    master_s2m_i.RespLast <= slaves_s2m(i).RespLast when CONV_INTEGER(active_port) = i else 'Z';
    master_s2m_i.Error <= slaves_s2m(i).Error when CONV_INTEGER(active_port) = i else 'Z';   
  end generate gen_s2m_ports;
  
  master_s2m <= master_s2m_i;
  
end mixed;
