--
-- ocp_dummy_target64.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- OCP Dummy Target
--
-- Dummy OCP Target module to legally respond to OCP transactions

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.ocpad.all;

entity ocp_dummy_target64 is
  
  port (
    -- OCP Interface
    ocp_clk : in std_logic;
    ocp_m2s : in ocpad_burst64_m2sT;
    ocp_s2m : out ocpad_burst64_s2mT);

end ocp_dummy_target64;

architecture rtl of ocp_dummy_target64 is

  signal rd : std_logic := '0';
  signal addr : std_logic_vector(21 downto 0);
  signal wr : std_logic := '0';
  signal write_reg : std_logic := '0';
  signal data : std_logic_vector(31 downto 0) := (others => '0');
  signal burst_length : ocp_BurstLengthT := (others => '0');
                                 
  signal wcmd : std_logic;
  signal rcmd : std_logic;

  signal SCmdAccept : std_logic := '0';
  signal SDataAccept : std_logic := '0';
  signal SResp : std_logic_vector(1 downto 0) := "00";
  signal SData : std_logic_vector(63 downto 0) := (others => '0');
  signal SRespLast  : std_logic := '0';
  signal SError : std_logic := '0';
  
    
  
begin  -- rtl

  process (ocp_clk)
  begin  -- process
    if ocp_clk'event and ocp_clk = '1' then  -- rising clock edge
      if OR_reduce(burst_length) = '1' then
        burst_length <= burst_length-1;
        addr <= addr + 8;
      else
        if ocp_m2s.Cmd = "001" or ocp_m2s.Cmd = "010" then
          if ocp_m2s.Cmd = "010" then
            burst_length <= ocp_m2s.BurstLength;
          end if;
          addr <= ocp_m2s.Addr(21 downto 0);
        end if;
      end if;

      rd <= OR_reduce(burst_length);

      if ocp_m2s.Cmd = "001" then
        write_reg <= '1';
      else
        if ocp_m2s.Cmd = "000" then
          write_reg <= '0';
        end if;
      end if;

      if ocp_m2s.DataValid = '1' then
        data <= ocp_m2s.Data(31 downto 0);
      end if;

      wr <= ocp_m2s.DataValid and (write_reg or wcmd) and not wr;

      SCmdAccept <= (wcmd or rcmd) and not SCmdAccept;
      if ocp_m2s.DataValid = '0' or ocp_m2s.DataValid = '1' then
        SDataAccept <= ocp_m2s.DataValid and not SDataAccept;
      end if;
      
      SData <= (others => '0');
      if rd = '1' then
        SResp <= "01";
      else
        SResp <= "00";
      end if;
      if OR_reduce(burst_length) = '0' and rd = '1' then
        SRespLast <= '1';
      else
        SRespLast <= '0';
      end if;
    end if;
  end process;

  wcmd <= '1' when ocp_m2s.Cmd = "001" else '0';
  rcmd <= '1' when ocp_m2s.Cmd = "010" else '0';

  
  ocp_s2m.CmdAccept       <=  SCmdAccept;      
  ocp_s2m.DataAccept      <=  SDataAccept;     
  ocp_s2m.Resp            <=  SResp;           
  ocp_s2m.Data            <=  SData;          
  ocp_s2m.RespLast        <=  SRespLast;       
  ocp_s2m.Error           <=  SError;       
  

 
  
end rtl;
