--
-- ocp_register_interface_r64.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- OCP Register Interface
--
-- Simple interface bridging from OCP to a register array


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.ocpad.all;

entity ocp_register_interface64 is
  generic (
    number_of_registers : natural := 1);
  port (
    -- OCP Interface
    ocp_clk : in std_logic;
    ocp_m2s : in ocpad_register64_m2sT;
    ocp_s2m : out ocpad_register64_s2mT;
    -- simple bus interface
    registers_out : out register64_array(0 to number_of_registers-1) := (others => (others => '0'));
    registers_back : in register64_array(0 to number_of_registers-1);
    register_written : out std_logic_vector(number_of_registers-1 downto 0) := (others => '0') 
    );

end ocp_register_interface64;

architecture rtl of ocp_register_interface64 is
                              
  signal SCmdAccept : std_logic := '0';
  signal SResp : std_logic_vector(1 downto 0) := "00";
  signal SData : std_logic_vector(63 downto 0) := (others => '0');

    
  
begin  -- rtl

  process (ocp_clk)
  begin  -- process
    if ocp_clk'event and ocp_clk = '1' then  -- rising clock edge
      register_written <= (others => '0');
      if ocp_m2s.Cmd = "001" then
        for i in 0 to number_of_registers-1 loop
          if ocp_m2s.Addr = CONV_STD_LOGIC_VECTOR(i*8,32) then
            register_written(i) <= '1';
            for j in 0 to 7 loop
              if ocp_m2s.DataByteEn(j) = '1' then
                registers_out(i)(j*8+7 downto j*8) <= ocp_m2s.Data(j*8+7 downto j*8);
              end if;              
            end loop;                   --j
          end if;
          if ocp_m2s.Addr = CONV_STD_LOGIC_VECTOR(i*8+4,32) then
            register_written(i) <= '1';
            for j in 0 to 3 loop
              if ocp_m2s.DataByteEn(j) = '1' then
                registers_out(i)(j*8+39 downto j*8+32) <= ocp_m2s.Data(j*8+7 downto j*8);
              end if;              
            end loop;                   --j
          end if;
        end loop;  -- i
      end if;
     
      if ocp_m2s.Cmd = "001"  or ocp_m2s.Cmd = "010" then
        SCmdAccept <= '1';
      else
        SCmdAccept <= '1';
      end if;

      if ocp_m2s.Cmd = "010" then
         for i in 0 to number_of_registers-1 loop
          if ocp_m2s.Addr = CONV_STD_LOGIC_VECTOR(i*8,32) then
            SData <= registers_back(i);
          end if;
        end loop;  -- i
        for i in 0 to number_of_registers-1 loop
          if ocp_m2s.Addr = CONV_STD_LOGIC_VECTOR(i*8+4,32) then
            SData <= EXT(registers_back(i)(63 downto 32),64);
          end if;
        end loop;  -- i
        SResp <= "01";
      else
        SResp <= "00";
      end if;
 
    end if;
  end process;

  
  ocp_s2m.CmdAccept       <=  SCmdAccept;      
  ocp_s2m.Resp            <=  SResp;
  ocp_s2m.Data            <=  SData; 
  
end rtl;
