--
-- ocp_slave_interface64.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- OCP Slave Bus Bridge
--
-- Simple interface bridging from OCP to a simple bus


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.ocpad.all;

entity ocp_slave_interface64 is
  generic (
    read_latency : integer := 1);
  port (
    -- OCP Interface
    ocp_clk : in std_logic;
    ocp_rst : in std_logic;
    ocp_m2s : in ocpad_register64_m2sT;
    ocp_s2m : out ocpad_register64_s2mT;
    -- simple bus interface
    simple_bus_addr : out std_logic_vector(21 downto 0);
    simple_bus_write : out std_logic;
    simple_bus_read : out std_logic;
    simple_bus_wdata : out std_logic_vector(31 downto 0);
    simple_bus_rdata : in std_Logic_vector(31 downto 0)
    );

end ocp_slave_interface64;

architecture rtl of ocp_slave_interface64 is

  signal rd : std_logic := '0';
  signal addr : std_logic_vector(21 downto 0);
  signal wr : std_logic := '0';
  signal write_reg : std_logic := '0';
  signal data : std_logic_vector(31 downto 0) := (others => '0');
  signal rd_req : std_Logic := '0'; 
                                 
  signal wcmd : std_logic;
  signal rcmd : std_logic;

  signal SCmdAccept : std_logic := '0';
  signal SResp : std_logic_vector(1 downto 0) := "00";
  signal SData : std_logic_vector(63 downto 0) := (others => '0');
 

  signal rd_pipe : std_logic_vector(read_latency downto 0);
    
  
begin  -- rtl

  process (ocp_clk)
  begin  -- process
    if ocp_clk'event and ocp_clk = '1' then  -- rising clock edge
      rd_req <= '0';
      if (ocp_m2s.Cmd = "001" or ocp_m2s.Cmd = "010") and rd_req = '0' then
        if ocp_m2s.Cmd = "010" then
           rd_req <= '1';
        end if;
        addr <= ocp_m2s.Addr(21 downto 0);
      end if;
     

      rd <= rd_req;

      rd_pipe <= rd_pipe(read_latency-1 downto 0) & rd_req;
      
      if ocp_m2s.Cmd = "001" then
        write_reg <= '1';
      else
        if ocp_m2s.Cmd = "000" then
          write_reg <= '0';
        end if;
      end if;

      if ocp_m2s.Cmd = "001" then
        data <= ocp_m2s.Data(31 downto 0);
      end if;

      wr <= to_std_logic(ocp_m2s.Cmd = "001") and (write_reg or wcmd) and not wr;

      SCmdAccept <= (wcmd or rcmd) and not SCmdAccept;
      SData <= EXT(simple_bus_rdata,64);

      SResp(0) <= rd_pipe(read_latency);
      SResp(1) <= '0';
 
    end if;
  end process;

  wcmd <= '1' when ocp_m2s.Cmd = "001" else '0';
  rcmd <= '1' when ocp_m2s.Cmd = "010" else '0';

  
  ocp_s2m.CmdAccept       <=  SCmdAccept;      
  ocp_s2m.Resp            <=  SResp;           
  ocp_s2m.Data            <=  SData;           
  

  simple_bus_addr <= addr;
  simple_bus_write <= wr;
  simple_bus_read <= rd;
  simple_bus_wdata <= data;
  
end rtl;
