--
-- ocpad_dataflow_components_pkg.vhd
-- Dataflow Header Package of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
--
library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.fifo.all;
use work.ocpad.all;

package ocpad_dataflow_components is

  component data_source_b32r32
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9;
      use_rate_control  : boolean := true;
      use_rfd           : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst32_m2sT;
      burst_s2m  : in  ocpad_burst32_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register32_m2sT;
      reg_s2m    : out ocpad_register32_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd        : in  std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_source_b32r64
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9;
      use_rate_control  : boolean := true;
      use_rfd           : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst32_m2sT;
      burst_s2m  : in  ocpad_burst32_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register64_m2sT;
      reg_s2m    : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd        : in  std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_source_b64r32
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9;
      use_rate_control  : boolean := true;
      use_rfd           : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst64_m2sT;
      burst_s2m  : in  ocpad_burst64_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register32_m2sT;
      reg_s2m    : out ocpad_register32_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd        : in  std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_source_b64r64
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9;
      use_rate_control  : boolean := true;
      use_rfd           : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst64_m2sT;
      burst_s2m  : in  ocpad_burst64_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register64_m2sT;
      reg_s2m    : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd        : in  std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;


  component data_source_b128r32
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9;
      use_rate_control  : boolean := true;
      use_rfd           : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst128_m2sT;
      burst_s2m  : in  ocpad_burst128_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register32_m2sT;
      reg_s2m    : out ocpad_register32_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd        : in  std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;


  component data_source_b128r64
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9;
      use_rate_control  : boolean := true;
      use_rfd           : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst128_m2sT;
      burst_s2m  : in  ocpad_burst128_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register64_m2sT;
      reg_s2m    : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd        : in  std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_source_2d_b128r64
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9;
      use_rate_control  : boolean := true;
      use_rfd           : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst128_m2sT;
      burst_s2m  : in  ocpad_burst128_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register64_m2sT;
      reg_s2m    : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd        : in  std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_sink_b32r32
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst32_m2sT;
      burst_s2m  : in  ocpad_burst32_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register32_m2sT;
      reg_s2m    : out ocpad_register32_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : in  std_logic_vector(data_width-1 downto 0);
      data_valid : in  std_logic;
      rfd        : out std_logic;
      flush_packet : in std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_sink_b32r64
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst32_m2sT;
      burst_s2m  : in  ocpad_burst32_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register64_m2sT;
      reg_s2m    : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : in  std_logic_vector(data_width-1 downto 0);
      data_valid : in  std_logic;
      rfd        : out std_logic;
      flush_packet : in std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_sink_b64r32
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst64_m2sT;
      burst_s2m  : in  ocpad_burst64_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register32_m2sT;
      reg_s2m    : out ocpad_register32_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : in  std_logic_vector(data_width-1 downto 0);
      data_valid : in  std_logic;
      rfd        : out std_logic;
      flush_packet : in std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_sink_b64r64
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst64_m2sT;
      burst_s2m  : in  ocpad_burst64_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register64_m2sT;
      reg_s2m    : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : in  std_logic_vector(data_width-1 downto 0);
      data_valid : in  std_logic;
      rfd        : out std_logic;
      flush_packet : in std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_sink_b128r32
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst128_m2sT;
      burst_s2m  : in  ocpad_burst128_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register32_m2sT;
      reg_s2m    : out ocpad_register32_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : in  std_logic_vector(data_width-1 downto 0);
      data_valid : in  std_logic;
      rfd        : out std_logic;
      flush_packet : in std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  component data_sink_b128r64
    generic (
      data_width        : integer := 16;
      data_shift        : integer := 16;
      shift_count_width : integer := 4;
      family            : family_t := family_virtex4;
      fifo_order        : integer := 9);
    port (
      -- Interface to OCP Burst Profile
      burst_clk  : in  std_logic;
      burst_m2s  : out ocpad_burst128_m2sT;
      burst_s2m  : in  ocpad_burst128_s2mT;
      -- Interface to OCP Register Profile
      reg_clk    : in  std_logic;
      reg_m2s    : in  ocpad_register64_m2sT;
      reg_s2m    : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk   : in  std_logic;
      data       : in  std_logic_vector(data_width-1 downto 0);
      data_valid : in  std_logic;
      rfd        : out std_logic;
      flush_packet : in std_logic;
      -- Interrupt out
      irq        : out std_logic);
  end component;

  

  
end ocpad_dataflow_components;
