--
-- Test Data Source
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library std;
use std.textio.all;

library work;
use work.ocpad.all;
use work.fifo.all;


entity test_data_source_2d is
end test_data_source_2d;

architecture behave of test_data_source_2d is

  constant rclk_period : time := 15 ns;
  constant bclk_period : time := 10 ns;
  constant dclk_period : time := 8 ns; 

  signal rclk,bclk,dclk : std_logic := '1';

  signal b_m2s : ocpad_burst128_m2sT;
  signal b_s2m : ocpad_burst128_s2mT;
  signal r_m2s : ocpad_register64_m2sT;
  signal r_s2m : ocpad_register64_s2mT;


  procedure ocpad_reg64_rst (
    signal   ocp_clk : in std_logic;    
    signal   reg64_m2s : out ocpad_register64_m2sT;
    signal   reg64_s2m : in  ocpad_register64_s2mT)
  is
    begin
      reg64_m2s.Reset_n <= '0';
      wait until ocp_clk ='1';
      wait until ocp_clk ='0';
      wait until ocp_clk ='1';
      wait until ocp_clk ='0';
      wait until ocp_clk ='1';
      reg64_m2s.Reset_n <= '1';
    end;
  
  procedure ocpad_reg64_write (
    signal   ocp_clk : in std_logic;    
    signal   reg64_m2s : out ocpad_register64_m2sT;
    signal   reg64_s2m : in  ocpad_register64_s2mT;
    constant data      : in  integer;
    constant addr      : in  integer)
  is
    begin
      wait until ocp_clk ='0';
      wait until ocp_clk ='1';
      reg64_m2s.Cmd <= ocp_Cmd_WRITE;
      reg64_m2s.Addr <= CONV_STD_LOGIC_VECTOR(addr,32);
      reg64_m2s.Data <= CONV_STD_LOGIC_VECTOR(data,64);
      reg64_m2s.DataByteEn <= (others => '1');
      reg64_m2s.Reset_n <= '1';
      if reg64_s2m.Cmdaccept = '0' then
        wait until reg64_s2m.CmdAccept = '1';
      end if;
      wait until ocp_clk ='0';
      wait until ocp_clk ='1';
      reg64_m2s.Cmd <= ocp_Cmd_IDLE;
    end;
  

  component ocp_dummy_target128
    port (
      -- OCP Interface
      ocp_clk : in std_logic;
      ocp_m2s : in ocpad_burst128_m2sT;
      ocp_s2m : out ocpad_burst128_s2mT);
  end component;

  component data_source_2d_b128r64
    generic (
      data_width : integer := 16;
      data_shift : integer := 16;
      shift_count_width : integer := 4;
      family     : family_t := family_virtex4;
      fifo_order : integer := 9;
      use_rate_control : boolean := true;
      use_rfd : boolean := true);
    port (
      -- Interface to OCP Burst Profile
      burst_clk : in  std_logic;
      burst_m2s : out ocpad_burst128_m2sT;
      burst_s2m : in  ocpad_burst128_s2mT;
      -- Interface to OCP Register Profile
      reg_clk  : in  std_logic;    
      reg_m2s  : in  ocpad_register64_m2sT;
      reg_s2m  : out ocpad_register64_s2mT;
      -- Interface to Data Flow
      data_clk : in  std_logic;
      data : out std_logic_vector(data_width-1 downto 0);
      data_valid : out std_logic;
      rfd : in std_logic;
      -- Interrupt out
      irq : out std_logic);
  end component;
  
  signal rfd : std_logic := '1';
  signal data : std_logic_vector(15 downto 0);
  signal data_valid : std_logic;
  
begin  -- behave

  rclk <= not rclk after rclk_period / 2;
  bclk <= not bclk after bclk_period / 2;
  dclk <= not dclk after dclk_period / 2;

  dut0: data_source_2d_b128r64
    generic map (
      data_width  => 16,
      data_shift => 16)
    port map (
      burst_clk  => bclk,
      burst_m2s  => b_m2s,
      burst_s2m  => b_s2m,
      reg_clk    => rclk,
      reg_m2s    => r_m2s,
      reg_s2m    => r_s2m,
      data_clk   => dclk,
      data       => data,
      data_valid => data_valid,
      rfd        => rfd);

  dummy0: ocp_dummy_target128
    port map (
      ocp_clk => bclk,
      ocp_m2s  => b_m2s,
      ocp_s2m => b_s2m);


  process
    begin
      -- Reset 
      ocpad_reg64_rst(rclk,r_m2s,r_s2m);
      -- Set Up Start Address
      ocpad_reg64_write(rclk,r_m2s,r_s2m,0,8*0);
      -- Set Up Word Count
      ocpad_reg64_write(rclk,r_m2s,r_s2m,1920*960/8,8*1);
      -- Set Up Rate Count
      ocpad_reg64_write(rclk,r_m2s,r_s2m,3,8*4);
      -- Set Up Interrupt Count
      ocpad_reg64_write(rclk,r_m2s,r_s2m,256,8*2);
      -- Set Up Pixel Limit
      ocpad_reg64_write(rclk,r_m2s,r_s2m,480/8,8*5);
      -- Set Up Row Limit
      ocpad_reg64_write(rclk,r_m2s,r_s2m,240,8*6);
      -- Set Up No Sections
      ocpad_reg64_write(rclk,r_m2s,r_s2m,4,8*7);
      -- Set Up Line Skip
      ocpad_reg64_write(rclk,r_m2s,r_s2m,3*480*2-(64*16-480*2),8*8);
      -- Set Up Section Skip
      ocpad_reg64_write(rclk,r_m2s,r_s2m,0-(64*16-480*2)-(1920*239*2),8*9);
      -- Set Up End Skip
      ocpad_reg64_write(rclk,r_m2s,r_s2m,0-(64*16-480*2),8*10);
      -- Set Up Start and end line masks
      ocpad_reg64_write(rclk,r_m2s,r_s2m,8+8*256,8*11); 
      -- Start
      ocpad_reg64_write(rclk,r_m2s,r_s2m,1,8*3);
      wait;
    end process;
    
  
end behave;
