--
-- test_ddma64-xrc4fx.vhd - Testbench for entity 'ddma64'
--
-- (C) Copyright Alpha Data 2007
--
-- Example testbench for:
--
--   o ADM-XRC-4FX
--
-- NOT SYNTHESIZABLE
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library std;
use std.textio.all;

library plxsim;
use plxsim.plxsim.all;

library work;
USE work.HY5PS121621F_PACK.all;
use work.memif.all;
use work.memif_def.all;
use work.memif_sim.all;

entity test_memory is
end test_memory;




architecture behav of test_memory is

  constant lclk_period : time := 15 ns;

  constant mclka_period : time := 4 ns;  

  constant priorities : integer_vector_t(0 to 2) := (2 => 1, others => 0);

  constant size : natural := 528;

  signal lclk     : std_logic                     := '1';
  signal lreset_l : std_logic                     := '1';
  signal lwrite   : std_logic                     := 'Z';
  signal lads_l   : std_logic                     := 'Z';
  signal lblast_l : std_logic                     := 'Z';
  signal lbterm_l : std_logic                     := 'Z';
  signal lad      : std_logic_vector(63 downto 0) := (others => 'Z');
  signal lready_l : std_logic                     := 'Z';
  signal lbe_l    : std_logic_vector(7 downto 0)  := (others => 'Z');
  signal ldreq_l  : std_logic_vector(3 downto 0)  := (others => 'Z');
  signal ldack_l  : std_logic_vector(3 downto 0)  := "1111";
  signal fholda   : std_logic                     := '0';
  signal lhold    : std_logic_vector(2 downto 0);
  signal lholda   : std_logic_vector(2 downto 0);

  signal mclka_p : std_logic := '0';
  signal mclka_n : std_logic := '1';

  signal clk200   : std_logic := '0';
  signal clk200_n : std_logic := '1';

  signal pcie_rx_p  : std_logic_vector(7 downto 0) := (others => 'H');
  signal pcie_rx_n  : std_logic_vector(7 downto 0) := (others => 'L');
  signal pcie_tx_p  : std_logic_vector(7 downto 0);
  signal pcie_tx_n  : std_logic_vector(7 downto 0);
  signal rear_rx_p  : std_logic_vector(3 downto 0) := (others => 'H');
  signal rear_rx_n  : std_logic_vector(3 downto 0) := (others => 'L');
  signal rear_tx_p  : std_logic_vector(3 downto 0);
  signal rear_tx_n  : std_logic_vector(3 downto 0);
  signal front_rx_p : std_logic_vector(3 downto 0) := (others => 'H');
  signal front_rx_n : std_logic_vector(3 downto 0) := (others => 'L');
  signal front_tx_p : std_logic_vector(3 downto 0);
  signal front_tx_n : std_logic_vector(3 downto 0);

  signal ddma_in0, ddma_in1          : locbus_ddma_in_t;
  signal ddma_out0, ddma_out1        : locbus_ddma_out_t := init_locbus_ddma_out;
  signal bus_in, bus_in0, bus_in1    : locbus_in_t;
  signal bus_out, bus_out0, bus_out1 : locbus_out_t      := init_locbus_out;

  signal logic0, logic1 : std_logic;

  constant bank0 : bank_t := (enable => true, ra_width => 19, rc_width => 22, rd_width => 32);
  constant bank1 : bank_t := (enable => true, ra_width => 19, rc_width => 22, rd_width => 32);
  
  signal ra0 : std_logic_vector(bank0.ra_width - 1 downto 0) := (others => 'Z');
  signal rc0 : std_logic_vector(bank0.rc_width - 1 downto 0) := (others => 'Z');
  signal rd0 : std_logic_vector(bank0.rd_width - 1 downto 0) := (others => 'Z');
  signal ra1 : std_logic_vector(bank1.ra_width - 1 downto 0) := (others => 'Z');
  signal rc1 : std_logic_vector(bank1.rc_width - 1 downto 0) := (others => 'Z');
  signal rd1 : std_logic_vector(bank1.rd_width - 1 downto 0) := (others => 'Z');
  signal ra0_ram : std_logic_vector(bank0.ra_width - 1 downto 0) := (others => 'Z');
  signal rc0_ram : std_logic_vector(bank0.rc_width - 1 downto 0) := (others => 'Z');
  signal rd0_ram : std_logic_vector(bank0.rd_width - 1 downto 0) := (others => 'Z');
  signal ra1_ram : std_logic_vector(bank1.ra_width - 1 downto 0) := (others => 'Z');
  signal rc1_ram : std_logic_vector(bank1.rc_width - 1 downto 0) := (others => 'Z');
  signal rd1_ram : std_logic_vector(bank1.rd_width - 1 downto 0) := (others => 'Z');
  
  
  
  procedure report_message(
    str : in string)
  is
    variable msg : line;
  begin
    write(msg, str);
    writeline(output, msg);
  end;

  procedure wait_cycles(
    constant n : in natural;
    signal   k : in std_logic)
  is
  begin
    for i in 0 to n - 1 loop
      wait until k'event and k = '1';
    end loop;
    wait for 1 ns;
  end;

  component memory
    generic(
      -- Memory bank parameters
      bank0             : in    bank_t := (enable => true, ra_width => 19, rc_width => 22, rd_width => 32);
      bank1             : in    bank_t := (enable => true, ra_width => 19, rc_width => 22, rd_width => 32));
    port(
      -- Local bus
      lclk              : in    std_logic;
      lreset_l          : in    std_logic; 
      lwrite            : in    std_logic;
      lads_l            : in    std_logic;
      lblast_l          : in    std_logic;
      lbe_l             : in    std_logic_vector(7 downto 0);
      lad               : inout std_logic_vector(63 downto 0);
      lbterm_l          : inout std_logic;
      lready_l          : inout std_logic; 
      fholda            : in    std_logic;
      ldreq_l  : out   std_logic_vector(3 downto 0);
      ldack_l  : in    std_logic_vector(3 downto 0);
      -- General purpose clock
      mclka_n           : in    std_logic;
      mclka_p           : in    std_logic;
      -- Reference clock
      refclk_n          : in    std_logic;
      refclk_p          : in    std_logic;
      -- Memory banks (up to 16 supported by this design)
      ra0               : out   std_logic_vector(bank0.ra_width - 1 downto 0);
      rc0               : inout std_logic_vector(bank0.rc_width - 1 downto 0);
      rd0               : inout std_logic_vector(bank0.rd_width - 1 downto 0);
      ra1               : out   std_logic_vector(bank1.ra_width - 1 downto 0);
      rc1               : inout std_logic_vector(bank1.rc_width - 1 downto 0);
      rd1               : inout std_logic_vector(bank1.rd_width - 1 downto 0));
  end component;


  component HY5PS121621F
    generic (
      TimingCheckFlag : boolean       := TRUE;
      PUSCheckFlag    : boolean       := false;
      Part_Number     : PART_NUM_TYPE := B667);
    port (DQ     : inout std_logic_vector(15 downto 0);
          LDQS  : inout std_logic;
          LDQSB : inout std_logic;
          UDQS  : inout std_logic;
          UDQSB : inout std_logic;
          LDM   : in    std_logic;
          WEB   : in    std_logic;
          CASB  : in    std_logic;
          RASB  : in    std_logic;
          CSB   : in    std_logic;
          BA    : in    std_logic_vector(1 downto 0);
          ADDR  : in    std_logic_vector(12 downto 0);
          CKE   : in    std_logic;
          CLK   : in    std_logic;
          CLKB  : in    std_logic;
          UDM   : in    std_logic);
  end component;




begin

  logic0 <= '0';
  logic1 <= '1';

  lclk <= not lclk after lclk_period / 2;

  mclka_p <= not mclka_p after mclka_period / 2;
  mclka_n <= not mclka_n after mclka_period / 2;

  clk200   <= not clk200   after 2.5 ns;
  clk200_n <= not clk200_n after 2.5 ns;

  reset_process : process
  begin
    lreset_l <= '0';
    wait for 150 ns;
    lreset_l <= '1';
    wait;
  end process;




  lads_l   <= 'H';
  lblast_l <= 'H';
  lbterm_l <= 'H';
  lready_l <= 'H';
  lwrite   <= 'H';

  stimulate_dd0 : process
    variable data      : byte_vector_t(0 to 127);
    variable be        : byte_enable_t(0 to 127);
    variable n         : natural;
    variable val32     : std_logic_vector(31 downto 0);
    variable tag       : natural;
    variable remaining : natural := size;
    variable chunk     : natural;
  begin
    wait for 180 us;
    wait_cycles(4, lclk);

    be    := (others => '1');
    tag   := 0;
    val32 := X"beef0000";
    while remaining /= 0 loop
      chunk := 88;
      if chunk > remaining then
        chunk := remaining;
      end if;

      for i in 0 to chunk / 4 - 1 loop
        data(4 * i to 4 * (i + 1) - 1) := conv_byte_vector(val32 + tag + i);
      end loop;

      plxsim_wait_demand(bus_in0, ddma_in0);
      plxsim_request_bus(true, bus_in0, bus_out0);
      plxsim_write_const_demand(3, X"00000000", be(0 to chunk - 1), data(0 to chunk - 1), n, bus_in0, bus_out0, ddma_in0, ddma_out0);
      plxsim_request_bus(false, bus_in0, bus_out0);

      remaining := remaining - n;
      tag       := tag + (n / 4);
    end loop;

    report_message("DDMA channel 0 process completed");

    wait;
  end process;

  stimulate_dd1 : process
    variable data                : byte_vector_t(0 to 127);
    variable be                  : byte_enable_t(0 to 127);
    variable n                   : natural;
    variable remaining           : natural := size;
    variable chunk               : natural;
    variable pat32, exp32, val32 : std_logic_vector(31 downto 0);
    variable tag                 : natural;
    variable num_err             : natural;
  begin
    wait for 180 us;
    wait_cycles(4000, lclk);

    be      := (others => '1');
    tag     := 0;
    pat32   := X"beef0000";
    num_err := 0;
    while remaining /= 0 loop
      chunk := 72;
      if chunk > remaining then
        chunk := remaining;
      end if;

      plxsim_wait_demand(bus_in1, ddma_in1);
      plxsim_request_bus(true, bus_in1, bus_out1);
      plxsim_read_const_demand(3, X"00000000", be(0 to chunk - 1), data(0 to chunk - 1), n, bus_in1, bus_out1, ddma_in1, ddma_out1);
      plxsim_request_bus(false, bus_in1, bus_out1);

      remaining := remaining - n;

      for i in 0 to n / 4 - 1 loop
        val32 := conv_std_logic_vector(data(4 * i to 4 * (i + 1) - 1));
        exp32 := pat32 + tag;
        if exp32 /= val32 then
          report_message("*** verification error: expected=0x" & conv_string_hex(exp32) &
                         " actual=0x" & conv_string_hex(val32));
          num_err := num_err + 1;
        end if;
        tag := tag + 1;
      end loop;
    end loop;

    report_message("DDMA channel 1 process completed with " & conv_string(num_err) & " data error(s)");

    wait;
  end process;

  stimulate_ds : process
    variable data : byte_vector_t(0 to 127);
    variable be   : byte_enable_t(0 to 127);
    variable n    : natural;
  begin
    wait for 180 us;

    wait_cycles(4, lclk);

    be := (others => '1');

    -- Set inbound and outbound length registers
    data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(size, 32));
    data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(size, 32));
    plxsim_request_bus(true, bus_in, bus_out);
    -- Test 2 non-burst OCP buses
    plxsim_write(3, true, X"00080000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);
    plxsim_write(3, true, X"00100000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);

    plxsim_read(3, true, X"00080000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);
    plxsim_read(3, true, X"00100000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);


    -- Set inbound and outbound length registers
    data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(size, 32));
    data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(size, 32));
    plxsim_write(3, true, X"00000000", be(0 to 7), data(0 to 7), n, bus_in, bus_out);

    -- Set burst size and target registers and row size register
    data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(16, 32));
    data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(16, 32));
    plxsim_write(3, true, X"00000030", be(0 to 7), data(0 to 7), n, bus_in, bus_out);
    -- Set n_rows and col_inc register
    data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(64, 32));
    data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(128, 32));
    plxsim_write(3, true, X"00000038", be(0 to 7), data(0 to 7), n, bus_in, bus_out);


    -- Set start addresss
    data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(0, 32));
    plxsim_write(2, true, X"00000040", be(0 to 3), data(0 to 3), n, bus_in, bus_out);
    plxsim_request_bus(false, bus_in, bus_out);

    wait for 20 us;


    plxsim_request_bus(true, bus_in, bus_out);

    -- Set burst size and target registers and row size register
    data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(16, 32));
    plxsim_write(2, true, X"00000044", be(0 to 3), data(0 to 3), n, bus_in, bus_out);
    data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(16, 32));
    plxsim_write(2, true, X"00000048", be(4 to 7), data(4 to 7), n, bus_in, bus_out);
    -- Set n_rows and col_inc register
    data(0 to 3) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(64, 32));
    plxsim_write(2, true, X"0000004C", be(0 to 3), data(0 to 3), n, bus_in, bus_out);
    data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(128, 32));
    plxsim_write(2, true, X"00000050", be(4 to 7), data(4 to 7), n, bus_in, bus_out);


    -- Set start addresss
    data(4 to 7) := conv_byte_vector(CONV_STD_LOGIC_VECTOR(0, 32));
    plxsim_write(2, true, X"00000054", be(4 to 7), data(4 to 7), n, bus_in, bus_out);


    plxsim_request_bus(false, bus_in, bus_out);

    wait;
  end process;

  UUT : memory
    port map(
      lclk      => lclk,
      lreset_l  => lreset_l,
      lwrite    => lwrite,
      lads_l    => lads_l,
      lblast_l  => lblast_l,
      lbterm_l  => lbterm_l,
      lad       => lad,
      lready_l  => lready_l,
      lbe_l     => lbe_l,
      ldack_l   => ldack_l,
      ldreq_l   => ldreq_l,
      fholda    => fholda,
      mclka_p   => mclka_p,
      mclka_n   => mclka_n,
      REFCLK_P  => CLK200,
      REFCLK_N  => CLK200_n,
      ra0        => ra0,
      rc0        => rc0,
      rd0        => rd0,
      ra1        => ra1,
      rc1        => rc1,
      rd1        => rd1);


  trace_model_0 : ddr2sdram_trace_model
    generic map(
      max_bank_width => 3,
      max_addr_width => 16,
      num_phys_bank => 1,
      dq_width => bank0.rd_width,
      k_width => 2,
      cke_width => 1,
      odt_width => 1,
      registered => false,
      t_trace_k => 0.5 ns,
      t_trace_ctl => 0.5 ns,
      t_trace_dm => 0.5 ns,
      t_trace_dq => 0.5 ns,
      t_trace_dqs => 0.5 ns)
    port map(
      mk => rc0(13 downto 12),
      mk_l => rc0(15 downto 14),
      mcke => rc0(16 downto 16),
      modt => rc0(17 downto 17),
      mwe_l => rc0(18),
      mcas_l => rc0(19),
      mras_l => rc0(20),
      mcs_l => rc0(21 downto 21),
      mba => ra0(18 downto 16),
      ma => ra0(15 downto 0),
      mdq => rd0,
      mdm => rc0(3 downto 0),
      mdqs => rc0(7 downto 4),
      mdqs_l => rc0(11 downto 8),
      dk => rc0_ram(13 downto 12),
      dk_l => rc0_ram(15 downto 14),
      dcke => rc0_ram(16 downto 16),
      dodt => rc0_ram(17 downto 17),
      dwe_l => rc0_ram(18),
      dcas_l => rc0_ram(19),
      dras_l => rc0_ram(20),
      dcs_l => rc0_ram(21 downto 21),
      dba => ra0_ram(18 downto 16),
      da => ra0_ram(15 downto 0),
      ddq => rd0_ram,
      ddm => rc0_ram(3 downto 0),
      ddqs => rc0_ram(7 downto 4),
      ddqs_l => rc0_ram(11 downto 8));

  mem_model_0 : for i in 0 to 1 generate
    chip0 : HY5PS121621F
      port map(
        DQ => rd0_ram(16 * (i + 1) - 1 downto 16 * i),
        LDM => rc0_ram(2 * i + 0),
        LDQS => rc0_ram(2 * i + 4),
        LDQSB => rc0_ram(2 * i + 8),
        UDM => rc0_ram(2 * i + 1),
        UDQS => rc0_ram(2 * i + 5),
        UDQSB => rc0_ram(2 * i + 9),
        WEB => rc0_ram(18),
        CASB => rc0_ram(19),
        RASB => rc0_ram(20),
        CSB => rc0_ram(21),
        BA => ra0_ram(17 downto 16),
        ADDR => ra0_ram(12 downto 0),
        CKE => rc0_ram(16),
        CLK => rc0_ram(i + 12),
        CLKB => rc0_ram(i + 14));
  end generate;

  trace_model_1 : ddr2sdram_trace_model
    generic map(
      max_bank_width => 3,
      max_addr_width => 16,
      num_phys_bank => 1,
      dq_width => bank1.rd_width,
      k_width => 2,
      cke_width => 1,
      odt_width => 1,
      registered => false,
      t_trace_k => 0.5 ns,
      t_trace_ctl => 0.5 ns,
      t_trace_dm => 0.5 ns,
      t_trace_dq => 0.5 ns,
      t_trace_dqs => 0.5 ns)
    port map(
      mk => rc1(13 downto 12),
      mk_l => rc1(15 downto 14),
      mcke => rc1(16 downto 16),
      modt => rc1(17 downto 17),
      mwe_l => rc1(18),
      mcas_l => rc1(19),
      mras_l => rc1(20),
      mcs_l => rc1(21 downto 21),
      mba => ra1(18 downto 16),
      ma => ra1(15 downto 0),
      mdq => rd1,
      mdm => rc1(3 downto 0),
      mdqs => rc1(7 downto 4),
      mdqs_l => rc1(11 downto 8),
      dk => rc1_ram(13 downto 12),
      dk_l => rc1_ram(15 downto 14),
      dcke => rc1_ram(16 downto 16),
      dodt => rc1_ram(17 downto 17),
      dwe_l => rc1_ram(18),
      dcas_l => rc1_ram(19),
      dras_l => rc1_ram(20),
      dcs_l => rc1_ram(21 downto 21),
      dba => ra1_ram(18 downto 16),
      da => ra1_ram(15 downto 0),
      ddq => rd1_ram,
      ddm => rc1_ram(3 downto 0),
      ddqs => rc1_ram(7 downto 4),
      ddqs_l => rc1_ram(11 downto 8));

  mem_model_1 : for i in 0 to 1 generate
    chip0 : HY5PS121621F
      port map(
        DQ => rd1_ram(16 * (i + 1) - 1 downto 16 * i),
        LDM => rc1_ram(2 * i + 0),
        LDQS => rc1_ram(2 * i + 4),
        LDQSB => rc1_ram(2 * i + 8),
        UDM => rc1_ram(2 * i + 1),
        UDQS => rc1_ram(2 * i + 5),
        UDQSB => rc1_ram(2 * i + 9),
        WEB => rc1_ram(18),
        CASB => rc1_ram(19),
        RASB => rc1_ram(20),
        CSB => rc1_ram(21),
        BA => ra1_ram(17 downto 16),
        ADDR => ra1_ram(12 downto 0),
        CKE => rc1_ram(16),
        CLK => rc1_ram(i + 12),
        CLKB => rc1_ram(i + 14));
  end generate;


  agent_ds : locbus_agent_mux64
    port map(
      lclk     => lclk,
      lreset_l => lreset_l,
      lads_l   => lads_l,
      lad      => lad,
      lbe_l    => lbe_l,
      lblast_l => lblast_l,
      lbterm_l => lbterm_l,
      lready_l => lready_l,
      lwrite   => lwrite,
      lhold    => lhold(2),
      lholda   => lholda(2),
      bus_in   => bus_in,
      bus_out  => bus_out);

  agent_dd0 : locbus_agent_mux64
    port map(
      lclk     => lclk,
      lreset_l => lreset_l,
      lads_l   => lads_l,
      lad      => lad,
      lbe_l    => lbe_l,
      lblast_l => lblast_l,
      lbterm_l => lbterm_l,
      lready_l => lready_l,
      lwrite   => lwrite,
      lhold    => lhold(0),
      lholda   => lholda(0),
      bus_in   => bus_in0,
      bus_out  => bus_out0);

  agent_dd1 : locbus_agent_mux64
    port map(
      lclk     => lclk,
      lreset_l => lreset_l,
      lads_l   => lads_l,
      lad      => lad,
      lbe_l    => lbe_l,
      lblast_l => lblast_l,
      lbterm_l => lbterm_l,
      lready_l => lready_l,
      lwrite   => lwrite,
      lhold    => lhold(1),
      lholda   => lholda(1),
      bus_in   => bus_in1,
      bus_out  => bus_out1);

  ddma_agent_0 : locbus_agent_ddma
    port map(
      ldreq_l => ldreq_l(0),
      ldack_l => ldack_l(0),
      dd_in   => ddma_in0,
      dd_out  => ddma_out0);

  ddma_agent_1 : locbus_agent_ddma
    port map(
      ldreq_l => ldreq_l(1),
      ldack_l => ldack_l(1),
      dd_in   => ddma_in1,
      dd_out  => ddma_out1);

  arb_0 : locbus_arb
    generic map(
      n_arb    => 3,
      priority => priorities)
    port map(
      lclk     => lclk,
      lreset_l => lreset_l,
      lhold    => lhold,
      lholda   => lholda);

  lbpcheck_0 : lbpcheck
    generic map(
      multiplexed => true,
      wide        => false)
    port map(
      lclk     => lclk,
      lreset_l => lreset_l,
      lads_l   => lads_l,
      l64_l    => lads_l,
      la       => lad(31 downto 2),
      lad_lo   => lad(31 downto 0),
      lad_hi   => lad(63 downto 32),
      lbe_lo_l => lbe_l(3 downto 0),
      lbe_hi_l => lbe_l(7 downto 4),
      lwrite   => lwrite,
      lblast_l => lblast_l,
      lready_l => lready_l,
      lbterm_l => lbterm_l);

  
  
end behav;
