--
-- ocp_ddma_components_b32.vhd
-- Header Package of ADPKGOCP Library
--
-- (C) 2007-2009 Alpha Data Ltd.


library IEEE;
use IEEE.std_logic_1164.all;

library work;
use work.fifo.all;
use work.ocpad.all;
use work.fifo_if_types.all;
use work.localbus_if_types.all;
use work.slave_bus_if_types.all;

package ocp_ddma_components is


  component ddma_rfsm_b32
    port (
      clk          : in  std_logic;
      -- burst parameters
      burst_size   : in  std_logic_vector(7 downto 0);
      xfer_size    : in  std_logic_vector(29 downto 0);
      row_size     : in  std_logic_vector(19 downto 0);
      n_rows       : in  std_logic_vector(19 downto 0);
      col_inc      : in  std_logic_vector(19 downto 0);
      start_addr   : in  std_logic_vector(31 downto 0);
      fgsg_size    : in  std_logic_vector(3 downto 0);
      fgsg_pattern : in  std_logic_vector(11 downto 0);
      -- control signals
      start        : in  std_logic;
      read_cmd     : out std_logic;
      cmd_addr     : out std_logic_vector(31 downto 0);
      cmd_accept   : in  std_logic;
      resp_valid   : in  std_logic;
      resp_accept  : out std_logic;
      burst_length : out std_logic_vector(7 downto 0);
      resp_last    : in  std_logic;
      fifo_level   : in  std_logic_vector(15 downto 0);
      fifo_wadv    : out std_logic;
      -- transfer status
      xfer_status  : out std_logic_vector(63 downto 0)
      );
  end component;

  component ddma_wfsm_b32
    port (
      clk          : in  std_logic;
      -- burst parameters
      burst_size   : in  std_logic_vector(7 downto 0);
      xfer_size    : in  std_logic_vector(29 downto 0);
      row_size     : in  std_logic_vector(19 downto 0);
      n_rows       : in  std_logic_vector(19 downto 0);
      col_inc      : in  std_logic_vector(19 downto 0);
      start_addr   : in  std_logic_vector(31 downto 0);
      -- control signals
      start        : in  std_logic;
      write_cmd    : out std_logic;
      cmd_addr     : out std_logic_vector(31 downto 0);
      cmd_accept   : in  std_logic;
      data_valid   : out std_logic;
      data_accept  : in  std_logic;
      burst_length : out std_logic_vector(7 downto 0);
      data_last    : out std_logic;
      fifo_level   : in  std_logic_vector(15 downto 0);
      fifo_radv    : out std_logic;
      -- transfer status
      xfer_status  : out std_logic_vector(63 downto 0)


      );
  end component;

  component ddma2ocp_rwmux_b32
    port (
      clk                : in  std_logic;
      rst                : in  std_logic;
      timeout_disable    : in std_logic;
      -- write_port
      write_cmd          : in  std_logic;
      write_addr         : in  std_logic_vector(31 downto 0);
      write_cmd_accept   : out std_logic;
      write_data         : in  std_logic_vector(31 downto 0);
      write_data_valid   : in  std_logic;
      write_data_accept  : out std_logic;
      write_burst_length : in  std_logic_vector(7 downto 0);
      write_data_last    : in  std_logic;
      -- read_port
      read_cmd           : in  std_logic;
      read_addr          : in  std_logic_vector(31 downto 0);
      read_cmd_accept    : out std_logic;
      read_data          : out std_logic_vector(31 downto 0);
      read_data_valid    : out std_logic;
      read_data_accept   : in  std_logic;
      read_burst_length  : in  std_logic_vector(7 downto 0);
      read_data_last     : out std_logic;
      -- OCP port
      ocp_bus_outputs    : out ocpad_burst32_m2sT := ocpad_burst32_default_m2s;
      ocp_bus_inputs     : in  ocpad_burst32_s2mT

      );
  end component;

  component ocp_ddma_engine_b32
     generic (
    number_of_ocp_masters : integer := 2);
  port (
    clk          : in  std_logic;
    rst          : in  std_logic;
    -- OCP Interfaces
    ocp_bus_outputs_array         : out ocpad_burst32_m2sT_array(0 to number_of_ocp_masters-1);
    ocp_bus_inputs_array          : in  ocpad_burst32_s2mT_array(0 to number_of_ocp_masters-1);
    -- FIFO Interface
    rfifo_inputs    : out  fifo_read_port_inputs_type;
    rfifo_outputs   : in   fifo_read_port_outputs_type;
    wfifo_inputs    : out  fifo_write_port_inputs_type;
    wfifo_outputs   : in   fifo_write_port_outputs_type;
    timeout_disable    : in std_logic;
    -- write side parameters
    write_burst_size   : in  std_logic_vector(7 downto 0);
    write_xfer_size    : in  std_logic_vector(29 downto 0);
    write_row_size     : in  std_logic_vector(19 downto 0);
    write_n_rows       : in  std_logic_vector(19 downto 0);
    write_col_inc      : in  std_logic_vector(19 downto 0);
    write_start_addr   : in  std_logic_vector(31 downto 0);
    write_start        : in  std_Logic;
    write_status       : out std_logic_vector(63 downto 0);
    write_target       : in std_logic_vector(2 downto 0);
    -- read side parameters
    read_burst_size   : in  std_logic_vector(7 downto 0);
    read_xfer_size    : in  std_logic_vector(29 downto 0);
    read_row_size     : in  std_logic_vector(19 downto 0);
    read_n_rows       : in  std_logic_vector(19 downto 0);
    read_col_inc      : in  std_logic_vector(19 downto 0);
    read_start_addr   : in  std_logic_vector(31 downto 0);
    read_start        : in  std_Logic;
    read_status       : out std_logic_vector(63 downto 0);
    read_target       : in std_logic_vector(2 downto 0);
    read_fgsg_size    : in  std_logic_vector(3 downto 0);
    read_fgsg_pattern : in  std_logic_vector(11 downto 0));
  end component;

  component localbus2ocp_bridge_r64b32
   generic (
    number_of_ocp_nonburst : integer := 7;
    number_of_ddma_engines : integer := 2;
    number_of_ocp_burst_per_engine : integer := 4;
    la_top           : natural := 23;
    family           : family_t := family_virtex4);
  port (
    lclk             : in    std_logic;  -- Local Bus Clock
    ocp_clk_nb       : in    std_logic;  -- Non-Burst Clock
    ocp_clk_b        : in    std_logic;  -- Burst Clock
    rst              : out   std_logic;  -- Global Reset (LRESET)
    -- OCP Interfaces
    ocp_nonburst_bus_outputs_array         : out ocpad_register64_m2sT_array(0 to number_of_ocp_nonburst-1);
    ocp_nonburst_bus_inputs_array          : in  ocpad_register64_s2mT_array(0 to number_of_ocp_nonburst-1);
    ocp_burst_bus_outputs_array         : out ocpad_burst32_m2sT_array(0 to number_of_ocp_burst_per_engine*number_of_ddma_engines-1);
    ocp_burst_bus_inputs_array          : in  ocpad_burst32_s2mT_array(0 to number_of_ocp_burst_per_engine*number_of_ddma_engines-1);
    -- Local Bus Interface
    localbus_inputs  : in  localbus_inputs_type;
    localbus_outputs : out localbus_outputs_type;
    system_status    : in  std_logic_vector(31 downto 0);
    system_irq       : in  std_logic_vector(31 downto 0);
    irq              : out std_logic);
  end component;

  component localbus2ocp_bridge_r32b32
   generic (
    number_of_ocp_nonburst : integer := 7;
    number_of_ddma_engines : integer := 2;
    number_of_ocp_burst_per_engine : integer := 4;
    la_top           : natural := 23;
    family           : family_t := family_virtex4);
  port (
    lclk             : in    std_logic;  -- Local Bus Clock
    ocp_clk_nb       : in    std_logic;  -- Non-Burst Clock
    ocp_clk_b        : in    std_logic;  -- Burst Clock
    rst              : out   std_logic;  -- Global Reset (LRESET)
    -- OCP Interfaces
    ocp_nonburst_bus_outputs_array         : out ocpad_register32_m2sT_array(0 to number_of_ocp_nonburst-1);
    ocp_nonburst_bus_inputs_array          : in  ocpad_register32_s2mT_array(0 to number_of_ocp_nonburst-1);
    ocp_burst_bus_outputs_array         : out ocpad_burst32_m2sT_array(0 to number_of_ocp_burst_per_engine*number_of_ddma_engines-1);
    ocp_burst_bus_inputs_array          : in  ocpad_burst32_s2mT_array(0 to number_of_ocp_burst_per_engine*number_of_ddma_engines-1);
    -- Local Bus Interface
    localbus_inputs  : in  localbus_inputs_type;
    localbus_outputs : out localbus_outputs_type;
    system_status    : in  std_logic_vector(31 downto 0);
    system_irq       : in  std_logic_vector(31 downto 0);
    irq              : out std_logic);
  end component;
  
end ocp_ddma_components;
