--
-- async_pulse.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2007-2009 Alpha Data Ltd.
--
-- async_pulse
--
-- Takes a single pulse across clock domains
--
-- For any rising edge on I, generates a 1 clock cycle pulse on O,
-- Using CLK.
-- Delayed by at least 1 CLK cycle

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

-- synthesis translate_off
library unisim;
use unisim.vcomponents.all;
-- synthesis translate_on


entity async_pulse is
  
  port (
    clk : in std_logic;
    I   : in std_Logic;
    O   : out std_Logic);

end async_pulse;

architecture rtl of async_pulse is

  signal latch,latch_reg,rst_latch : std_logic := '0';

begin  -- rtl

  -- Latch rising edge of I
  process (I, rst_latch)
  begin  -- process
    if rst_latch = '1' then             -- asynchronous reset (active low)
      latch <= '0';
    elsif I'event and I = '1' then  -- rising clock edge
      latch <= '1';
    end if;
  end process;


  process (clk)
  begin  -- process
    if clk'event and clk = '1' then  -- rising clock edge
      latch_reg <= latch;
      O <= latch_reg;
    end if;
  end process;

  rst_latch <= latch and latch_reg;
  
end rtl;
