--
-- localbus_io_iobufs.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2007-2009 Alpha Data Ltd.
--
--
-- Component for mapping localbus_if record data types
-- to local bus pin mappings
-- Adds tri-state logic
--
--
-- Created 5/10/2007 : AM - Alpha Data
--
library IEEE;
use IEEE.std_logic_1164.all;
library work;
use work.localbus_if_types.all;
library unisim;
use unisim.vcomponents.all;

entity localbus_io is
  generic (
    bus_width    : integer := 64;
    dma_channels : integer := 4);
  port (
    localbus_inputs  : out localbus_inputs_type;
    localbus_outputs : in  localbus_outputs_type;
    lreset_l         : in    std_logic;
    lwrite           : in    std_logic;
    lads_l           : in    std_logic;
    lblast_l         : in    std_logic;
    lbterm_l         : inout std_logic;
    lad              : inout std_logic_vector(bus_width-1 downto 0);
    lready_l         : inout std_logic;
    lbe_l            : in    std_logic_vector(bus_width/8-1 downto 0);
    fholda           : in    std_logic;
    ldreq_l          : out   std_logic_vector(dma_channels-1 downto 0);
    ldack_l          : in    std_logic_vector(dma_channels-1 downto 0));

end localbus_io;

architecture rtl of localbus_io is

  signal lad_i : std_logic_vector(bus_width-1 downto 0);
  
begin  -- rtl

  assert bus_width = 64 or bus_width =32 report "Bus Width must be 32 or 64 bits";
  assert dma_channels = 2 or dma_channels = 4  report "DMA Channels must be 2 or 4";
  
  localbus_inputs.lreset_l <= lreset_l;
  localbus_inputs.lwrite <= lwrite;
  localbus_inputs.lads_l <= lads_l;
  localbus_inputs.lblast_l <= lblast_l;
  
  localbus_inputs.la_i <= lad_i(31 downto 0);
  gen_32bit_bus_inputs: if bus_width = 32 generate
    localbus_inputs.lad_i(63 downto 32) <= (others => '0');
    localbus_inputs.lbe_l(7 downto 4) <= (others => '1');
  end generate gen_32bit_bus_inputs;
  localbus_inputs.lad_i(bus_width-1 downto 0) <= lad_i;
  localbus_inputs.lbe_l(bus_width/8-1 downto 0) <= lbe_l;
  localbus_inputs.fholda <= fholda;
  gen_2dack_inputs: if dma_channels = 2 generate
    localbus_inputs.ldack_l(3 downto 2) <= (others => '1');
  end generate gen_2dack_inputs;
  localbus_inputs.ldack_l(dma_channels-1 downto 0) <= ldack_l;

  ldreq_l <= localbus_outputs.ldreq_l(dma_channels-1 downto 0);

  lbterm_iobuf: IOBUF
    port map (
      O  => localbus_inputs.lbterm_l_i,
      IO => lbterm_l,
      I  => localbus_outputs.lbterm_l_o,
      T  => localbus_outputs.lbterm_l_t);
 
  lready_iobuf: IOBUF
    port map (
      O  => open,
      IO => lready_l,
      I  => localbus_outputs.lready_l_o ,
      T  => localbus_outputs.lready_l_t);  
    
  gen_lad: for i in 0 to bus_width-1 generate
    lad_iobuf_i: IOBUF
      port map (
        O  => lad_i(i),
        IO => lad(i),
        I  => localbus_outputs.lad_o(i),
        T  => localbus_outputs.lad_t(i));
  end generate gen_lad;

  

end rtl;
