--
-- slave_bus_if_types.vhd
-- Header Package of ADPKGOCP Library
--
-- (C) 2007-2009 Alpha Data Ltd.
--
--
-- Basic data types for simple non-bursting slave bus
--

library IEEE;
use IEEE.std_logic_1164.all;

library work;
use work.ocpad.all;

package slave_bus_if_types is

  type slave_bus_outputs_type is record
    rd : std_logic;
    wr : std_logic;
    a  : std_logic_vector(21 downto 0);
    d  : std_logic_vector(63 downto 0);
    be : std_logic_vector(7 downto 0);
  end record;

  type slave_bus_inputs_type is record
    q  : std_logic_vector(63 downto 0);
    qv : std_logic;
  end record;

  type slave_bus_outputs_array is array (natural range <>) of slave_bus_outputs_type;
  type slave_bus_inputs_array is array (natural range <>) of slave_bus_inputs_type;



  type register_array is array (natural range <>) of std_logic_vector(31 downto 0);

  component slavebus_register_bank
    generic (
      number_of_control_registers : integer := 4;
      control_base_address        : integer := 0;    -- in bytes
      number_of_status_registers  : integer := 4;
      status_base_address         : integer := 32);  -- in bytes
    port (
      clk                     : in  std_logic;
      slave_bus_in            : in  slave_bus_outputs_type;
      slave_bus_out           : out slave_bus_inputs_type;
      control_register_values : out register_array(0 to number_of_control_registers-1);
      control_register_writes : out std_logic_vector(number_of_control_registers-1 downto 0);
      status_register_values  : in  register_array(0 to number_of_status_registers-1);
      status_register_reads_done : out std_logic_vector(number_of_status_registers-1 downto 0));
  end component;

  component slavebus2ocp64
    port (
      lclk                     : in  std_logic;
      ocp_clk                     : in  std_logic;
      rst                     : in  std_logic;
      slave_bus_outputs       : in  slave_bus_outputs_type;
      slave_bus_inputs        : out slave_bus_inputs_type;
      ocp_bus_outputs         : out ocpad_register64_m2sT := ocpad_register64_default_m2s;
      ocp_bus_inputs          : in  ocpad_register64_s2mT;
      page_register           : in  std_logic_vector(31 downto 0));  
  end component;

  component slavebus2ocp32
    port (
      lclk                     : in  std_logic;
      ocp_clk                     : in  std_logic;
      rst                     : in  std_logic;
      slave_bus_outputs       : in  slave_bus_outputs_type;
      slave_bus_inputs        : out slave_bus_inputs_type;
      ocp_bus_outputs         : out ocpad_register32_m2sT := ocpad_register32_default_m2s;
      ocp_bus_inputs          : in  ocpad_register32_s2mT;
      page_register           : in  std_logic_vector(31 downto 0));  
  end component;

  component slavebus_mux
    generic (
      number_of_output_buses : integer := 2); 
    port (
      clk                        : in  std_logic;
      slave_bus_outputs          : in  slave_bus_outputs_type;
      slave_bus_inputs           : out slave_bus_inputs_type;
      slave_bus_multiple_outputs : out slave_bus_outputs_array(0 to number_of_output_buses-1);
      slave_bus_multiple_inputs  : in  slave_bus_inputs_array(0 to number_of_output_buses-1));
  end component;

  component slavebus_rom
    port (
      clk   : in std_logic;
      slave_bus_in            : in  slave_bus_outputs_type;
      slave_bus_out           : out slave_bus_inputs_type;
      rom_addressed           : out std_logic);
  end component;
  
end package slave_bus_if_types;


