--
-- slavebus2ocp64.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2007-2009 Alpha Data Ltd.
--
--
-- Simple slavebus to OCP bridge
--
-- Converts simple slave bus to non-bursting OCP bus

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

-- synthesis translate_off
library unisim;
use unisim.vcomponents.all;
-- synthesis translate_on

library work;
use work.slave_bus_if_types.all;
use work.ocpad.all;

entity slavebus2ocp64 is

  port (
    lclk                     : in  std_logic;
    ocp_clk                     : in  std_logic;
    rst                     : in  std_logic;
    slave_bus_outputs       : in  slave_bus_outputs_type;
    slave_bus_inputs        : out slave_bus_inputs_type;
    ocp_bus_outputs         : out ocpad_register64_m2sT := ocpad_register64_default_m2s;
    ocp_bus_inputs          : in  ocpad_register64_s2mT;
    page_register           : in  std_logic_vector(31 downto 0));

end slavebus2ocp64;

architecture rtl of slavebus2ocp64 is

  signal rd,wr,qv : std_logic;

  component async_pulse
    port (
      clk : in std_logic;
      I   : in std_Logic;
      O   : out std_Logic);
  end component;

  signal reset_n : std_logic;
  
begin  -- rtl

  ap0: async_pulse 
    port map (
      clk => ocp_clk,
      I   => slave_bus_outputs.rd,
      O   => rd);

  ap1: async_pulse 
    port map (
      clk => ocp_clk,
      I   => slave_bus_outputs.wr,
      O   => wr);

  ap2: async_pulse 
    port map (
      clk => lclk,
      I   => qv,
      O   => slave_bus_inputs.qv);

  
  process (ocp_clk)
  begin  -- process
    if ocp_clk'event and ocp_clk = '1' then  -- rising clock edge
      reset_n <= not rst;
      ocp_bus_outputs.reset_n <= reset_n;
      if rd = '1' then
        ocp_bus_outputs.cmd <= "010";
      elsif wr = '1' then
        ocp_bus_outputs.cmd <= "001";
      elsif ocp_bus_inputs.cmdaccept = '1' then
        ocp_bus_outputs.cmd <= "000";
      end if;
 
      if wr = '1' or rd = '1' then
        ocp_bus_outputs.addr <= page_register(31 downto 19) & slave_bus_outputs.a(18 downto 0);
      end if;
      
      if wr = '1' then
        ocp_bus_outputs.data <= slave_bus_outputs.d;
        ocp_bus_outputs.databyteen <= slave_bus_outputs.be;
      end if;
      
      if ocp_bus_inputs.resp = "01" and qv = '0'then
        -- Receive just first data word 
        qv <= '1';
        slave_bus_inputs.q <= ocp_bus_inputs.data;
      else
        qv <= '0';
      end if;       
    end if;
  end process;
 
  
  
  
end rtl;
