--
-- slavebus_rom.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2007-2009 Alpha Data Ltd.
--
--
-- ROM memory on slave bus
-- For storing FPGA information
-- ROM should be initialised using data2mem
--
-- Always located at top of 512k Slavebus window
-- Occupies top 512x32 words

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library unisim;
use unisim.vcomponents.all;

library work;
use work.slave_bus_if_types.all;

entity slavebus_rom is
 
  port (
    clk   : in std_logic;
    slave_bus_in            : in  slave_bus_outputs_type;
    slave_bus_out           : out slave_bus_inputs_type;
    rom_addressed           : out std_logic);

end slavebus_rom;

architecture rtl of slavebus_rom is

  signal addr : std_logic_vector(8 downto 0) := (others => '0');
  signal do : std_logic_vector(31 downto 0);

  signal qv : std_logic := '0';
  signal q : std_logic_vector(63 downto 0);
  signal rd : std_logic;
  signal rd_queue : std_logic_vector(2 downto 0) := (others => '0');
  
begin  -- rtl

  info_rom0: RAMB16_S36
    port map (
      we   => '0',
      en   => '1',
      ssr => '0',
      clk  => clk,
      addr => addr,
      di   => X"00000000",
      dip  => X"0",
      do   => do,
      dop  => open);

  rd <= slave_bus_in.rd and AND_reduce(slave_bus_in.a(18 downto 11));
  
  rd_rom: process (clk)
  begin  -- process rd_rom
    if clk'event and clk = '1' then  -- rising clock edge
      rd_queue <= rd_queue(1 downto 0) & rd;
      if rd = '1' then
        addr <= slave_bus_in.a(10 downto 2);
      else
        addr <= addr+1;
      end if;

      if rd_queue(1) = '1' then
        q(31 downto 0) <= do;
      end if;
      if rd_queue(2) = '1' then
        q(63 downto 32) <= do;
      end if;
      qv <= rd_queue(2);
      rom_addressed <= rd or OR_reduce(rd_queue);
    end if;
  end process rd_rom;


  

  slave_bus_out.qv <= qv;
  slave_bus_out.q <= q;
  
end rtl;
