--
-- infer_sdpbram.vhd
-- Memory Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- Dual Port RAM FIFO
--
-- Configurable data width
--
-- Use behavioural BRAM description to allow tools to identify
-- and use DPBRAMs appropriately
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;

entity infer_sdpbram is
  
  generic (
    width : integer := 64);

  port (
    wclk   : in  std_logic;
    rclk   : in  std_logic;
    wadv   : in  std_logic;
    wdata  : in  std_logic_vector(width-1 downto 0);
    waddr  : in  std_logic_vector(8 downto 0) := (others => '0');
    raddr  : in  std_logic_vector(8 downto 0) := (others => '0');
    rdata  : out std_logic_vector(width-1 downto 0));

end infer_sdpbram;

architecture rtl of infer_sdpbram is

  type ram_type is array (511 downto 0) of std_logic_vector(width-1 downto 0);
  signal ram : ram_type;
  
begin  -- rtl

  -- Model Block Rams
  process (wclk)
  begin  -- process
    if wclk'event and wclk = '1' then  -- rising clock edge
      if wadv = '1' then
        ram(conv_integer(waddr)) <= wdata;
      end if;
    end if;
  end process;
  
  process (rclk)
  begin  -- process
    if rclk'event and rclk = '1' then  -- rising clock edge
      rdata <= ram(conv_integer(raddr));
    end if;
  end process;
  
 
end rtl;

