--
-- ocpad_memif_pkg.vhd
-- Memory Header Package of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- Component Definitions for OCPAD Memory Interface Components
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.ocpad.all;


package ocpad_memif is

  component infer_sdpbram
    generic (
      width : integer := 64);
    port (
      wclk  : in  std_logic;
      rclk  : in  std_logic;
      wadv  : in  std_logic;
      wdata : in  std_logic_vector(width-1 downto 0);
      waddr : in  std_logic_vector(8 downto 0) := (others => '0');
      raddr : in  std_logic_vector(8 downto 0) := (others => '0');
      rdata : out std_logic_vector(width-1 downto 0));
  end component;


  component ocp2memif128
    generic (
      use_chipscope : boolean := false;
      addr_width    : integer := 28);
    port (
      clk     : in  std_logic;
      -- OCP Port
      ocp_m2s : in  ocpad_burst128_m2sT;
      ocp_s2m : out ocpad_burst128_s2mT;
      -- MemIF Port (128 bits wide)
      sr      : out std_logic;
      ce      : out std_logic;
      tag     : out std_logic_vector(1 downto 0);
      w       : out std_logic;
      a       : out std_logic_vector(addr_width-1 downto 0);
      d       : out std_logic_vector(127 downto 0);
      be      : out std_logic_vector(15 downto 0);
      q       : in  std_logic_vector(127 downto 0);
      qtag    : in  std_logic_vector(1 downto 0);
      valid   : in  std_logic;
      ready   : in  std_logic);
  end component;

  component ocp2memif64
    generic (
      use_chipscope : boolean := false;
      addr_width    : integer := 28);
    port (
      clk     : in  std_logic;
      -- OCP Port
      ocp_m2s : in  ocpad_burst64_m2sT;
      ocp_s2m : out ocpad_burst64_s2mT;
      -- MemIF Port (64 bits wide)
      sr      : out std_logic;
      ce      : out std_logic;
      tag     : out std_logic_vector(1 downto 0);
      w       : out std_logic;
      a       : out std_logic_vector(addr_width-1 downto 0);
      d       : out std_logic_vector(63 downto 0);
      be      : out std_logic_vector(7 downto 0);
      q       : in  std_logic_vector(63 downto 0);
      qtag    : in  std_logic_vector(1 downto 0);
      valid   : in  std_logic;
      ready   : in  std_logic);
  end component;

  component ocp2memif32
    generic (
      use_chipscope : boolean := false;
      addr_width    : integer := 28);
    port (
      clk     : in  std_logic;
      -- OCP Port
      ocp_m2s : in  ocpad_burst32_m2sT;
      ocp_s2m : out ocpad_burst32_s2mT;
      -- MemIF Port (64 bits wide)
      sr      : out std_logic;
      ce      : out std_logic;
      tag     : out std_logic_vector(1 downto 0);
      w       : out std_logic;
      a       : out std_logic_vector(addr_width-1 downto 0);
      d       : out std_logic_vector(31 downto 0);
      be      : out std_logic_vector(3 downto 0);
      q       : in  std_logic_vector(31 downto 0);
      qtag    : in  std_logic_vector(1 downto 0);
      valid   : in  std_logic;
      ready   : in  std_logic);
  end component;

  component ocp_bram_interface32
    generic (
      read_latency : integer := 1);
    port (
      -- OCP Interface
      ocp_clk    : in  std_logic;
      ocp_rst    : in  std_logic;
      ocp_m2s    : in  ocpad_register32_m2sT;
      ocp_s2m    : out ocpad_register32_s2mT;
      -- simple bus interface
      bram_addr  : out std_logic_vector(15 downto 0);
      bram_write : out std_logic;
      bram_read  : out std_logic;
      bram_be    : out std_logic_vector(3 downto 0);
      bram_wdata : out std_logic_vector(31 downto 0);
      bram_rdata : in  std_logic_vector(31 downto 0)
      );

  end component;

  component ocp_bram_interface64
    generic (
      read_latency : integer := 1);
    port (
      -- OCP Interface
      ocp_clk    : in  std_logic;
      ocp_rst    : in  std_logic;
      ocp_m2s    : in  ocpad_register64_m2sT;
      ocp_s2m    : out ocpad_register64_s2mT;
      -- simple bus interface
      bram_addr  : out std_logic_vector(15 downto 0);
      bram_write : out std_logic;
      bram_read  : out std_logic;
      bram_be    : out std_logic_vector(7 downto 0);
      bram_wdata : out std_logic_vector(63 downto 0);
      bram_rdata : in  std_logic_vector(63 downto 0)
      );
  end component;

  component ocpsimmem128
    generic (
      initialise_dwidth : integer := 0;
      initialise_file   : boolean := false;
      input_file        : string  := "";
      addr_width        : integer := 16;
      latency           : integer := 1);
    port (
      clk         : in  std_logic;
      -- OCP Port
      ocp_m2s     : in  ocpad_burst128_m2sT;
      ocp_s2m     : out ocpad_burst128_s2mT;
      -- Save Data
      save        : in  boolean;
      save_start  : in  integer;
      save_end    : in  integer;
      output_file : in  string := "");
  end component;


  component ocpsimmem64
    generic (
      initialise_dwidth : integer := 0;
      initialise_file   : boolean := false;
      input_file        : string  := "";
      addr_width        : integer := 16;
      latency           : integer := 1);
    port (
      clk         : in  std_logic;
      -- OCP Port
      ocp_m2s     : in  ocpad_burst64_m2sT;
      ocp_s2m     : out ocpad_burst64_s2mT;
      -- Save Data
      save        : in  boolean;
      save_start  : in  integer;
      save_end    : in  integer;
      output_file : in  string := "");
  end component;

  component ocpsimmem32
    generic (
      initialise_dwidth : integer := 0;
      initialise_file   : boolean := false;
      input_file        : string  := "";
      addr_width        : integer := 16;
      latency           : integer := 1);
    port (
      clk         : in  std_logic;
      -- OCP Port
      ocp_m2s     : in  ocpad_burst32_m2sT;
      ocp_s2m     : out ocpad_burst32_s2mT;
      -- Save Data
      save        : in  boolean;
      save_start  : in  integer;
      save_end    : in  integer;
      output_file : in  string := "");
  end component;
  
end ocpad_memif;
