--
-- ocpad_profile_bridge_pkg.vhd
-- Header Package of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- Component Definitions for OCPAD Profile to Profile Bridges
--

library ieee;
use ieee.std_logic_1164.all;

library work;
use work.ocpad.all;
use work.fifo.all;

package ocpad_profile_bridge is

  component profile_bridge_b32b32
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst32_m2sT;
      master_s2m : out ocpad_burst32_s2mT;
      slave_m2s  : out ocpad_burst32_m2sT;
      slave_s2m  : in  ocpad_burst32_s2mT);
  end component;

  component profile_bridge_b64b64
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst64_m2sT;
      master_s2m : out ocpad_burst64_s2mT;
      slave_m2s  : out ocpad_burst64_m2sT;
      slave_s2m  : in  ocpad_burst64_s2mT);
  end component;

  component profile_bridge_b128b128
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst128_m2sT;
      master_s2m : out ocpad_burst128_s2mT;
      slave_m2s  : out ocpad_burst128_m2sT;
      slave_s2m  : in  ocpad_burst128_s2mT);
  end component;

  component profile_bridge_b32b64
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst32_m2sT;
      master_s2m : out ocpad_burst32_s2mT;
      slave_m2s  : out ocpad_burst64_m2sT;
      slave_s2m  : in  ocpad_burst64_s2mT);
  end component;

  component profile_bridge_b64b32
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst64_m2sT;
      master_s2m : out ocpad_burst64_s2mT;
      slave_m2s  : out ocpad_burst32_m2sT;
      slave_s2m  : in  ocpad_burst32_s2mT);
  end component;

  component profile_bridge_b32b128
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst32_m2sT;
      master_s2m : out ocpad_burst32_s2mT;
      slave_m2s  : out ocpad_burst128_m2sT;
      slave_s2m  : in  ocpad_burst128_s2mT);
  end component;

  component profile_bridge_b128b32
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst128_m2sT;
      master_s2m : out ocpad_burst128_s2mT;
      slave_m2s  : out ocpad_burst32_m2sT;
      slave_s2m  : in  ocpad_burst32_s2mT);
  end component;

  component profile_bridge_b128b64
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst128_m2sT;
      master_s2m : out ocpad_burst128_s2mT;
      slave_m2s  : out ocpad_burst64_m2sT;
      slave_s2m  : in  ocpad_burst64_s2mT);
  end component;

  component profile_bridge_b64b128
    generic (
      family     : family_t := family_virtex4;
      fifo_order : integer := 4);
    port (
      master_clk : in  std_logic;
      slave_clk  : in  std_logic;
      master_m2s : in  ocpad_burst64_m2sT;
      master_s2m : out ocpad_burst64_s2mT;
      slave_m2s  : out ocpad_burst128_m2sT;
      slave_s2m  : in  ocpad_burst128_s2mT);
  end component;

  component profile_bridge_r32b32
    generic (
      burst_length : integer := 1);
    port (
      clk : in  std_logic;
      master_m2s : in  ocpad_register32_m2sT;
      master_s2m : out ocpad_register32_s2mT;
      slave_m2s  : out ocpad_burst32_m2sT;
      slave_s2m  : in  ocpad_burst32_s2mT);
  end component;
  
  component profile_bridge_r64b64
    generic (
      burst_length : integer := 1);
    port (
      clk : in  std_logic;
      master_m2s : in  ocpad_register64_m2sT;
      master_s2m : out ocpad_register64_s2mT;
      slave_m2s  : out ocpad_burst64_m2sT;
      slave_s2m  : in  ocpad_burst64_s2mT);
  end component;
  
  component profile_bridge_r64r32
    generic (
      burst_length : integer := 1);
    port (
      clk : in  std_logic;
      master_m2s : in  ocpad_register64_m2sT;
      master_s2m : out ocpad_register64_s2mT;
      slave_m2s  : out ocpad_register32_m2sT;
      slave_s2m  : in  ocpad_register32_s2mT);
  end component;
  
  component profile_bridge_r32r64
    generic (
      burst_length : integer := 1);
    port (
      clk : in  std_logic;
      master_m2s : in  ocpad_register32_m2sT;
      master_s2m : out ocpad_register32_s2mT;
      slave_m2s  : out ocpad_register64_m2sT;
      slave_s2m  : in  ocpad_register64_s2mT);
  end component;  
  
end ocpad_profile_bridge;
