--
-- profile_bridge_r32r64.vhd
-- Component of ADPKGOCP Library
--
-- (C) 2008-2009 Alpha Data Ltd.
--
-- Profile to Profile Synchronous Bridge
--
-- R32 to R64
--
-- Connects an R32 master to an R64 slave profile
-- Synchronous Design
-- Generates single word transfer
--


library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_misc.all;

library work;
use work.fifo.all;
use work.ocpad.all;

entity profile_bridge_r32r64 is
  
  generic (
    burst_length : integer := 1);
  port (
    clk : in  std_logic;
    master_m2s : in  ocpad_register32_m2sT;
    master_s2m : out ocpad_register32_s2mT;
    slave_m2s  : out ocpad_register64_m2sT;
    slave_s2m  : in  ocpad_register64_s2mT);

end profile_bridge_r32r64;

architecture mixed of profile_bridge_r32r64 is

  signal rst_i : std_logic;
  signal msb : std_logic;

  
begin  -- mixed

  rst_i <= not master_m2s.Reset_n;

  slave_m2s.Cmd <= master_m2s.Cmd;
  slave_m2s.Addr <= master_m2s.Addr(31 downto 3) & "000";
  slave_m2s.Data <= master_m2s.Data & X"00000000" when master_m2s.Addr(2) = '1' else X"00000000" & master_m2s.Data;
  slave_m2s.Reset_n <= master_m2s.Reset_n;
  slave_m2s.DataByteEn <= master_m2s.DataByteEn & "0000" when master_m2s.Addr(2) = '1' else "0000" & master_m2s.DataByteEn;
  
  master_s2m.CmdAccept <= slave_s2m.CmdAccept;
  master_s2m.Resp <= slave_s2m.Resp;
  master_s2m.Data <= slave_s2m.Data(31 downto 0) when msb = '0' else slave_s2m.Data(63 downto 32);
  
  convert_profile: process (clk, rst_i)
  begin  -- process convert_profile
    if rst_i = '1' then                 -- asynchronous reset
      msb <= '0';
    elsif clk'event and clk = '1' then  -- rising clock edge
      if master_m2s.Cmd = ocp_Cmd_READ then
        msb <= master_m2s.Addr(2);
      end if; 
    end if;
  end process convert_profile;
  
end mixed;
