
#include "lb2ocp.h"

PLB2OCP LB2OCP_Create(ADMXRC2_HANDLE card, int use64bit_dma, int use64bit_registers)
{
    PLB2OCP ret_val = NULL;
    ADMXRC2_STATUS		status;
    ADMXRC2_SPACE_INFO	spInfo;
    ADMXRC2_CARD_INFO	cardInfo;
    DWORD	config;
    int i;
    
    
    ret_val = (PLB2OCP) malloc(sizeof(LB2OCP));
    
    if (ret_val!=NULL)
    {
      ret_val->num_dma_engines = 0;
      
      for (i=0;i<4;i++)
      {
        ret_val->burst_size[i] = 32;
        ret_val->row_skip[i] = 0;
        ret_val->col_skip[i] = 0;
        ret_val->n_rows[i] = 65536;
      }
      ret_val->ier = 0;
      ret_val->card = card;
      
      status = ADMXRC2_GetSpaceInfo(card, 0, &spInfo);
	    if (status == ADMXRC2_SUCCESS) {
	      ret_val->fpgaSpace = (volatile DWORD*) spInfo.VirtualBase;
	      status = ADMXRC2_GetCardInfo(card, &cardInfo);
	      
	      ret_val->dmamode = ADMXRC2_BuildDMAModeWord(
		      cardInfo.BoardType,
		      use64bit_dma ? ADMXRC2_IOWIDTH_64 : ADMXRC2_IOWIDTH_32,
		      0,
		      ADMXRC2_DMAMODE_FIXEDLOCAL |
		      ADMXRC2_DMAMODE_USEREADY |
		      ADMXRC2_DMAMODE_USEBTERM |
		      ADMXRC2_DMAMODE_BURSTENABLE |
		      ADMXRC2_DMAMODE_DEMAND);
	      
        config = use64bit_registers ? ADMXRC2_SPACE_WIDTH_64 : ADMXRC2_SPACE_WIDTH_32;
		    config = ADMXRC2_SPACE_SET_WIDTH | config;
		    status = ADMXRC2_SetSpaceConfig(card, 0, config);
	    } 
	    else
	    {
  	    free(ret_val);    	
  		  ret_val = NULL;
	    }
    }   
    return ret_val;
}

void LB2OCP_Destroy(PLB2OCP bridge)
{
    free(bridge);
}

void LB2OCP_SetNumberDMAEngines(PLB2OCP bridge, unsigned int num_dma_engines)
{
    bridge->num_dma_engines = num_dma_engines;
}

int LB2OCP_SetBurstSize(PLB2OCP bridge, unsigned int channel, unsigned int size)
{
  if ((channel<2*bridge->num_dma_engines)&&(size<256))
  {
    bridge->burst_size[channel] = size;
    return 1;
  }
  else
    return 0;
}

int LB2OCP_SetRowSkip(PLB2OCP bridge, unsigned int channel, unsigned int size)
{
  if ((channel<2*bridge->num_dma_engines))
  {
    bridge->row_skip[channel] = size;
    return 1;
  }
  else
    return 0;
}

int LB2OCP_SetColumnSkip(PLB2OCP bridge, unsigned int channel, unsigned int size)
{
  if ((channel<2*bridge->num_dma_engines))
  {
    bridge->col_skip[channel] = size;
    return 1;
  }
  else
    return 0;
}

int LB2OCP_SetNRows(PLB2OCP bridge, unsigned int channel, unsigned int size)
{
  if ((channel<2*bridge->num_dma_engines))
  {
    bridge->n_rows[channel] = size;
    return 1;
  }
  else
    return 0;
}

int LB2OCP_TranferDataToFPGA(PLB2OCP bridge, 
                       unsigned int engine,
                       unsigned int target,
                       unsigned int size,
                       unsigned int ocp_start_addr,
                       unsigned int host_offset,
                       ADMXRC2_DMADESC bufHandle)
{
  ADMXRC2_STATUS		status;
  HANDLE				event;
  int timeout=0;  

  if (engine<bridge->num_dma_engines)
  {
    event = CreateEvent(NULL, TRUE, FALSE, NULL);
    if (engine==0)
    {
      bridge->fpgaSpace[LB2OCPB_DDMA_CH0_XFER_SIZE] = size;
      bridge->fpgaSpace[LB2OCPB_DENG0_WBURSTLEN] = (target<<8) | bridge->burst_size[0];
      bridge->fpgaSpace[LB2OCPB_DENG0_WROWSIZE] = (int) bridge->burst_size[0] + bridge->row_skip[0];
      bridge->fpgaSpace[LB2OCPB_DENG0_WNROWS] = bridge->n_rows[0];
      bridge->fpgaSpace[LB2OCPB_DENG0_WCOLINC] = (int) bridge->burst_size[0] + bridge->row_skip[0] + bridge->col_skip[0];
      bridge->fpgaSpace[LB2OCPB_DENG0_WSTARTADDR] = ocp_start_addr;    
    }
    else
    {
      bridge->fpgaSpace[LB2OCPB_DDMA_CH2_XFER_SIZE] = size;
      bridge->fpgaSpace[LB2OCPB_DENG1_WBURSTLEN] = (target<<8) | bridge->burst_size[2];
      bridge->fpgaSpace[LB2OCPB_DENG1_WROWSIZE] = (int) bridge->burst_size[2] + bridge->row_skip[2];
      bridge->fpgaSpace[LB2OCPB_DENG1_WNROWS] = (int) bridge->n_rows[2];
      bridge->fpgaSpace[LB2OCPB_DENG1_WCOLINC] = bridge->burst_size[2] + bridge->row_skip[2] + bridge->col_skip[2];
      bridge->fpgaSpace[LB2OCPB_DENG1_WSTARTADDR] = ocp_start_addr;  
    }
    status = ADMXRC2_DoDMA(
			 bridge->card,
			 bufHandle,
			 host_offset,
			 size,
			 0x40,
			 ADMXRC2_PCITOLOCAL,
			 0+2*engine,
			 bridge->dmamode,
			 0,
			 NULL,
			 event);
    
    if (engine==0)
      timeout = (bridge->fpgaSpace[LB2OCPB_DENG0_WSTATUS]&8)>>3; 
    else
      timeout = (bridge->fpgaSpace[LB2OCPB_DENG1_WSTATUS]&8)>>3; 

		CloseHandle(event);
  }
  
  if (timeout)
    return 0;  
  else
    return (status==ADMXRC2_SUCCESS);
  
}


int LB2OCP_TranferDataToHost(PLB2OCP bridge, 
                       unsigned int engine,
                       unsigned int target,
                       unsigned int size,
                       unsigned int ocp_start_addr,
                       unsigned int host_offset,
                       ADMXRC2_DMADESC bufHandle)
{
  ADMXRC2_STATUS		status;
  HANDLE				event;
  int timeout=0;  

  if (engine<bridge->num_dma_engines)
  {
    event = CreateEvent(NULL, TRUE, FALSE, NULL);
    if (engine==0)
    {
      bridge->fpgaSpace[LB2OCPB_DDMA_CH1_XFER_SIZE] = size;
      bridge->fpgaSpace[LB2OCPB_DENG0_RBURSTLEN] = (target<<8) | bridge->burst_size[1];
      bridge->fpgaSpace[LB2OCPB_DENG0_RROWSIZE] = (int) bridge->burst_size[1] + bridge->row_skip[1];
      bridge->fpgaSpace[LB2OCPB_DENG0_RNROWS] = bridge->n_rows[1];
      bridge->fpgaSpace[LB2OCPB_DENG0_RCOLINC] = (int) bridge->burst_size[1] + bridge->row_skip[1] + bridge->col_skip[1];
      bridge->fpgaSpace[LB2OCPB_DENG0_RSTARTADDR] = ocp_start_addr;    
    }
    else
    {
      bridge->fpgaSpace[LB2OCPB_DDMA_CH3_XFER_SIZE] = size;
      bridge->fpgaSpace[LB2OCPB_DENG1_RBURSTLEN] = (target<<8) | bridge->burst_size[3];
      bridge->fpgaSpace[LB2OCPB_DENG1_RROWSIZE] = (int) bridge->burst_size[3] + bridge->row_skip[3];
      bridge->fpgaSpace[LB2OCPB_DENG1_RNROWS] = bridge->n_rows[3];
      bridge->fpgaSpace[LB2OCPB_DENG1_RCOLINC] = (int) bridge->burst_size[3] + bridge->row_skip[3] + bridge->col_skip[3];
      bridge->fpgaSpace[LB2OCPB_DENG1_RSTARTADDR] = ocp_start_addr;  
    }
    status = ADMXRC2_DoDMA(
			 bridge->card,
			 bufHandle,
			 host_offset,
			 size,
			 0x40,
			 ADMXRC2_LOCALTOPCI,
			 1+2*engine,
			 bridge->dmamode,
			 0,
			 NULL,
			 event);
    
    if (engine==0)
      timeout = (bridge->fpgaSpace[LB2OCPB_DENG0_RSTATUS]&8)>>3; 
    else
      timeout = (bridge->fpgaSpace[LB2OCPB_DENG1_RSTATUS]&8)>>3; 

		CloseHandle(event);
  }

  if (timeout)
    return 0;  
  else
    return (status==ADMXRC2_SUCCESS);
  
}


DWORD LB2OCP_ReadRegister(PLB2OCP bridge, unsigned int space, unsigned int offset)
{
  if ((space>0)&&(space<8))
    return bridge->fpgaSpace[ space*0x020000 | (offset&0x1FFFF)];
  else
    return 0xDEADFFFF;
}

DWORD LB2OCP_ReadRegisterPaged(PLB2OCP bridge, unsigned int space, unsigned int offset)
{
  if ((space>0)&&(space<8)) {
    bridge->fpgaSpace[LB2OCPB_REG_SPACE1_PAGE-1+space] = (offset&0x3FFE0000)<<2;
    return bridge->fpgaSpace[space*0x020000 | (offset&0x1FFFF)];
  }
  else
    return 0xDEADFFFF;
}

void LB2OCP_WriteRegister(PLB2OCP bridge, unsigned int space, unsigned int offset, DWORD data)
{
  if ((space>0)&&(space<8))
    bridge->fpgaSpace[space*0x020000 | (offset&0x1FFFF)] = data;
}

void LB2OCP_WriteRegisterPaged(PLB2OCP bridge, unsigned int space, unsigned int offset, DWORD data)
{
  if ((space>0)&&(space<8)) {
    bridge->fpgaSpace[LB2OCPB_REG_SPACE1_PAGE-1+space] = (offset&0x3FFE0000)<<2;
    bridge->fpgaSpace[space*0x020000 | (offset&0x1FFFF)] = data;
  } 
}

DWORD LB2OCP_ReadInfoRom(PLB2OCP bridge, unsigned int offset)
{
  if (offset<512)
    return bridge->fpgaSpace[INFO_ROM_BASE+offset];
  else
    return 0xDEADFFFF;
  
}

void LB2OCP_EnableInterrupts(PLB2OCP bridge, unsigned int mask)
{
   bridge->ier = bridge->ier | mask;
   bridge->fpgaSpace[LB2OCPB_IER] = bridge->ier; 
}

void LB2OCP_DisableInterrupts(PLB2OCP bridge, unsigned int mask)
{
   bridge->ier = bridge->ier & ~mask;
   bridge->fpgaSpace[LB2OCPB_IER] = bridge->ier; 
}

void LB2OCP_ClearInterrupts(PLB2OCP bridge, unsigned int mask)
{
   bridge->fpgaSpace[LB2OCPB_ICR] = mask; 
}

int LB2OCP_TestInterrupts(PLB2OCP bridge, unsigned int mask)
{
   return (bridge->fpgaSpace[LB2OCPB_ISR]&mask) ? 1 : 0; 
}

int LB2OCP_Status(PLB2OCP bridge)
{
   return (bridge->fpgaSpace[LB2OCPB_SYSTEM_STATUS]); 
}
