/*
  LB2OCP.H


*/
#ifndef LB2OCP_H

#include <admxrc2.h>

#define LB2OCP_H

typedef enum {
  LB2OCPB_DDMA_CH0_XFER_SIZE=0,
  LB2OCPB_DDMA_CH1_XFER_SIZE=1,
  LB2OCPB_DDMA_CH2_XFER_SIZE=2,
  LB2OCPB_DDMA_CH3_XFER_SIZE=3,

  LB2OCPB_REG_SPACE1_PAGE=5,
  LB2OCPB_REG_SPACE2_PAGE=6,
  LB2OCPB_REG_SPACE3_PAGE=7,
  LB2OCPB_REG_SPACE4_PAGE=8,
  LB2OCPB_REG_SPACE5_PAGE=9,
  LB2OCPB_REG_SPACE6_PAGE=10,
  LB2OCPB_REG_SPACE7_PAGE=11,

  LB2OCPB_DENG0_WBURSTLEN =12,
  LB2OCPB_DENG0_WROWSIZE  =13,
  LB2OCPB_DENG0_WNROWS    =14,
  LB2OCPB_DENG0_WCOLINC   =15,
  LB2OCPB_DENG0_WSTARTADDR=16,
  LB2OCPB_DENG0_RBURSTLEN =17,
  LB2OCPB_DENG0_RROWSIZE  =18,
  LB2OCPB_DENG0_RNROWS    =19,
  LB2OCPB_DENG0_RCOLINC   =20,
  LB2OCPB_DENG0_RSTARTADDR=21,
  LB2OCPB_DENG1_WBURSTLEN =22,
  LB2OCPB_DENG1_WROWSIZE  =23,
  LB2OCPB_DENG1_WNROWS    =24,
  LB2OCPB_DENG1_WCOLINC   =25,
  LB2OCPB_DENG1_WSTARTADDR=26,
  LB2OCPB_DENG1_RBURSTLEN =27,
  LB2OCPB_DENG1_RROWSIZE  =28,
  LB2OCPB_DENG1_RNROWS    =29,
  LB2OCPB_DENG1_RCOLINC   =30,
  LB2OCPB_DENG1_RSTARTADDR=31,

  LB2OCPB_IER=32,
  LB2OCPB_ICR=33,
  LB2OCPB_SYSTEM_STATUS=34,
  LB2OCPB_ISR=35,
  LB2OCPB_DDMA_CH0_XFER_LEFT=36,
  LB2OCPB_DDMA_CH1_XFER_LEFT=37,
  LB2OCPB_DDMA_CH2_XFER_LEFT=38,
  LB2OCPB_DDMA_CH3_XFER_LEFT=39,
  LB2OCPB_DENG0_WSTATUS=40,
  LB2OCPB_DENG0_RSTATUS=42,
  LB2OCPB_DENG1_WSTATUS=44,
  LB2OCPB_DENG1_RSTATUS=46,
  INFO_ROM_BASE=(0x7f800>>2)
} LB2OCP_REG_ADDRESS;

typedef struct _LB2OCP {
  ADMXRC2_HANDLE card;
  volatile DWORD *fpgaSpace;
  
  unsigned int num_dma_engines;
  unsigned int burst_size[4];  
  int row_skip[4];
  int col_skip[4];
  unsigned int n_rows[4];
  
  DWORD dmamode;
  DWORD ier;
  
} LB2OCP;

typedef LB2OCP* PLB2OCP;


PLB2OCP LB2OCP_Create(ADMXRC2_HANDLE card, int use64bit_dma, int use64bit_registers);

void LB2OCP_Destroy(PLB2OCP bridge);

void LB2OCP_SetNumberDMAEngines(PLB2OCP bridge, unsigned int num_dma_engines);

int LB2OCP_SetBurstSize(PLB2OCP bridge, unsigned int channel, unsigned int size);

int LB2OCP_SetRowSkip(PLB2OCP bridge, unsigned int channel, unsigned int size);

int LB2OCP_SetColumnSkip(PLB2OCP bridge, unsigned int channel, unsigned int size);

int LB2OCP_SetNRows(PLB2OCP bridge, unsigned int channel, unsigned int size);

int LB2OCP_TranferDataToFPGA(PLB2OCP bridge, 
                       unsigned int engine,
                       unsigned int target,
                       unsigned int size,
                       unsigned int ocp_start_addr,
                       unsigned int host_offset,
                       ADMXRC2_DMADESC bufhandle);
                       
int LB2OCP_TranferDataToHost(PLB2OCP bridge, 
                       unsigned int engine,
                       unsigned int target,
                       unsigned int size,
                       unsigned int ocp_start_addr,
                       unsigned int host_offset,
                       ADMXRC2_DMADESC bufhandle);  
            
DWORD LB2OCP_ReadRegister(PLB2OCP bridge, unsigned int space, unsigned int offset);

DWORD LB2OCP_ReadRegisterPaged(PLB2OCP bridge, unsigned int space, unsigned int offset);  

void LB2OCP_WriteRegister(PLB2OCP bridge, unsigned int space, unsigned int offset, DWORD data);

void LB2OCP_WriteRegisterPaged(PLB2OCP bridge, unsigned int space, unsigned int offset, DWORD data);

DWORD LB2OCP_ReadInfoRom(PLB2OCP bridge, unsigned int offset);                                    
     

void LB2OCP_EnableInterrupts(PLB2OCP bridge, unsigned int mask);

void LB2OCP_DisableInterrupts(PLB2OCP bridge, unsigned int mask);

void LB2OCP_ClearInterrupts(PLB2OCP bridge, unsigned int mask);

int LB2OCP_TestInterrupts(PLB2OCP bridge, unsigned int mask);

int LB2OCP_Status(PLB2OCP bridge);
                  
#endif
