
#include "ocpsink.h"

POCPSINK OCPSINK_Create(PLB2OCP bridge, 
                        unsigned int burst_engine,
                        unsigned int burst_target,
                        unsigned int reg_bank,
                        unsigned int reg_offset,
                        unsigned int irq_mask,
                        unsigned int use64bit)
{
  POCPSINK ret_val = NULL;
  
  ret_val = (POCPSINK) malloc(sizeof(OCPSINK));
  if ((ret_val!=NULL)&&(bridge!=NULL))
  {
    ret_val->reg_bank = reg_bank;
    ret_val->burst_engine = burst_engine;
    ret_val->burst_target = burst_target;
    ret_val->fpgaSpace = (volatile DWORD *)  ( (unsigned int) bridge->fpgaSpace + 0x80000*reg_bank + reg_offset );
    ret_val->irq_mask = irq_mask;
    ret_val->use64bit = use64bit;
  }
  return ret_val;
}

void OCPSINK_Destroy(POCPSINK sink)
{
  free(sink);
}

int OCPSINK_StartCapture(POCPSINK sink, 
                          unsigned int start_addr,
                          unsigned int word_limit,
                          unsigned int irq_limit,
                          unsigned int continuous)
{
  int running=0;
  int sc64bit;
   
  if (sink->use64bit)
    sc64bit = 2;
  else
    sc64bit = 1;
       
  running = sink->fpgaSpace[OCPSINK_CONTROL*sc64bit]&1;             
 
  if (running)
  {
    return 0;
  }   
  else
  {
    sink->fpgaSpace[OCPSINK_START_ADDR*sc64bit] = start_addr;
    sink->fpgaSpace[OCPSINK_WORD_LIMIT*sc64bit] = word_limit;
    sink->fpgaSpace[OCPSINK_IRQ_LIMIT*sc64bit] = irq_limit;
    sink->fpgaSpace[OCPSINK_CONTROL*sc64bit] = 4;
    if (continuous)
      sink->fpgaSpace[OCPSINK_CONTROL*sc64bit] = 3;
    else
      sink->fpgaSpace[OCPSINK_CONTROL*sc64bit] = 1; 
    
    return 1;
  }
}

void OCPSINK_HaltContinuous(POCPSINK sink) 
{
  int sc64bit;
   
  if (sink->use64bit)
    sc64bit = 2;
  else
    sc64bit = 1;
  
  sink->fpgaSpace[OCPSINK_CONTROL*sc64bit] = 0; 
}


void OCPSINK_Clear(POCPSINK sink) 
{
  int sc64bit;
   
  if (sink->use64bit)
    sc64bit = 2;
  else
    sc64bit = 1;
  
  sink->fpgaSpace[OCPSINK_CONTROL*sc64bit] = 4; 
 
}

int OCPSINK_IsRunning(POCPSINK sink) 
{
  int sc64bit;
   
  if (sink->use64bit)
    sc64bit = 2;
  else
    sc64bit = 1;
    
  return (sink->fpgaSpace[OCPSINK_CONTROL*sc64bit])&1;            
}

unsigned int OCPSINK_GetCurrentAddress(POCPSINK sink) 
{
    int sc64bit;
   
  if (sink->use64bit)
    sc64bit = 2;
  else
    sc64bit = 1;
    
  return sink->fpgaSpace[OCPSINK_CURRENT_ADDR*sc64bit];
}

unsigned int OCPSINK_GetCurrentDataCount(POCPSINK sink) 
{
  int sc64bit;
   
  if (sink->use64bit)
    sc64bit = 2;
  else
    sc64bit = 1;
    
  return sink->fpgaSpace[OCPSINK_CURRENT_WCOUNT*sc64bit];
}

unsigned int OCPSINK_GetCurrentIntCount(POCPSINK sink) 
{
  int sc64bit;
   
  if (sink->use64bit)
    sc64bit = 2;
  else
    sc64bit = 1;
    
  return sink->fpgaSpace[OCPSINK_CURRENT_ICOUNT*sc64bit];
}